/********************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  OPM ADC-1/2 test#1 for TORNADO-6x/P6x/E6x.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


   Description:
   ------------
	This demo performs DAQ test for ADC-1/2 in the OPM DAQ mode
	as the following:
	  - on-board PFG 65MHz is used as sampling frequecy source
	  - ADC A0/B0 analog inputs are used as input signals. Recommended
	    external analog inputs are 1MHz sine signals.
	  - S/W synchronization is used for ADC DAQ start-up
	  - ADC FIFO PAF flag is used as DAQ TF
	  - download ADC FIFO data by DSP s/w and check for correct number of words
	    written and correct functionality of ADC-1/2 FIFO FF/PAF/EF/PAE flags
	  - ADC-1/2 FIFO data are transferred into C6x DSP on-chip memory and 
	    can be veiwed via Code Composer signal viewer in time domain (only
	    first 0x1000 samples of FIFO data are transfered into DSP on-chip RAM):
		- ADC-1 data are available at 0x80008000 address
		- ADC-2 data are available at 0x8000C000 address
	

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board. TI CCS tools are required to 
	recompile this demo.


  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/P6x mainboard
***********************************************************************/


#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#include	"math.h"
#include	"c6x_def.h"

//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#include			"te6x_dsp.h"
#define _REMOVE_PIOX_RESET	TE6X_set_pxsx_run(0x3)
#define	_PIOX16_SADDR_		TE6X_PIOX_SADDR
#endif

#if(_T6X_SELECTED)
#include			"t6x_dsp.h"
#define _REMOVE_PIOX_RESET	T6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		T6X_PIOX_SADDR
#endif

#if(_TP6X_SELECTED)
#include			"tp6x_dsp.h"
#define _REMOVE_PIOX_RESET	TP6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		TP6X_PIOX_SADDR
#endif
//--------------------------------------------------------------

// define host DSP platform for PA65_HD.H header file
#define _HOST_C6X_DSP_		1

//~~~~~~~~~~ include T/PDAS DCM header file ~~~~~~~~~~~~~~~~~~~~
#include	"pa65_hd.h"	
#include	"pa65x_hd.h"	
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



// perform host TORNADO specific EMIF initialization
init_cpu()
{
#if(_E6X_SELECTED)
	*C6X_EMIF_GCR=TE6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TE6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TE6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TE6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TE6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TE6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TE6X_EMIF_CE3_DATA;
#endif

#if(_T6X_SELECTED)
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif

#if(_TP6X_SELECTED)
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);
#endif

	_REMOVE_PIOX_RESET;	// release PIOX reset
}



	// set PFG to 65MHz = (25MHz*2*(V=130)/((R=50)*(S=2)*(X=1))
#define PFG_V_DATA		130
#define PFG_R_DATA      	50
#define PFG_S_DATA		TPDAS_A65M_S_DIV2
#define PFG_X_DATA		TPDAS_A65M_X_DIV1

	// define ADC input channel
#define	ADC_IMUX		TPDAS_A65M_ADC_IMUX_CH0

	// FIFO PAE/PAF flags offset
#define	_ADC_FIFO_PAE_OFFSET	0x04000
#define	_ADC_FIFO_PAF_OFFSET	0x08000

	// pointers for saving ADC-1/2 FIFO data and data length
#define	_SRAM_SADDR_ADC1	(C6201_ONCHIP_DRAM_SADDR + (2*(C6201_ONCHIP_DRAM_LEN/4)))
#define	_SRAM_SADDR_ADC2	(C6201_ONCHIP_DRAM_SADDR + (3*(C6201_ONCHIP_DRAM_LEN/4)))
#define	_SRAM_DATA_SAMPLES	0x1000L


#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


main()
{ int i;
  int *mptr1, *mptr2, fifo_data;

	init_cpu();

		// configure PFG, Fs selector and ADC DAQ SYNC
	TPDAS_A65M_SET_FS_SEL_PFG;
	TPDAS_A65M_SET_ADC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_SYNC_MODE_MASTER;

	TPDAS_A65M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	TPDAS_A65M_set_daq_mux_rg(ADC_IMUX);
	TPDAS_A65M_SET_DSIN_SEL_ADC;
	TPDAS_A65M_SET_ADC_FMT_12;


        		// now reset ADC FIFO and program FIFO PAF/PAE flags
	if(   TPDAS_A65M_set_adc_fifo_offset(_ADC_FIFO_PAE_OFFSET,_ADC_FIFO_PAF_OFFSET)!=0
          )
            goto error;


			// set ADC FIFO DAQ mode with PAF termination flag
			// and initialize DAQ procedure
	TPDAS_A65M_SET_ADC_MODE_OPM;
	TPDAS_A65M_SET_ADC_DAQ_TF_SEL_PAF;
	TPDAS_A65M_RESET_ADC_DAQ;         // perform FIFO reset ADC DAQ control

	if(   TPDAS_A65M_adc1_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc1_fifo_pae_data() ==0
	   || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc1_fifo_paf_data() !=0
           || TPDAS_A65M_adc2_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc2_fifo_pae_data() ==0
	   || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc2_fifo_paf_data() !=0
          )
            goto error;

        TPDAS_A65M_SET_ADC_DAQ_RUN;

	if(   TPDAS_A65M_adc_daq_run_data() == 0
	   || TPDAS_A65M_adc_daq_end_data() != 0
	  ) goto error;
	

			// wait until ADC DAQ terminates
	while(TPDAS_A65M_adc_daq_end_data() == 0);


		// now read FIFO data, store it in DPRAM,
		// and verify number of written FIFO samples.
	mptr1 = (int *) _SRAM_SADDR_ADC1;
	mptr2 = (int *) _SRAM_SADDR_ADC2;

	for(i=0;i<_ADC_FIFO_PAF_OFFSET; i++)
	  {
			// check for FIFO PAF/EF/PAE flags function perfectly
	    if(   TPDAS_A65M_adc1_fifo_ef_data()!=0
	       || TPDAS_A65M_adc2_fifo_ef_data()!=0
               || (   i>=(_ADC_FIFO_PAF_OFFSET-_ADC_FIFO_PAE_OFFSET+1L)
	       	   && (   TPDAS_A65M_adc1_fifo_pae_data()==0  
		       || TPDAS_A65M_adc2_fifo_pae_data()==0
		      )
                  )
			// !!!! PAF flag is cleared after the 2nd read cycle
	       || (i>2 &&  (    TPDAS_A65M_adc1_fifo_paf_data()!=0 
			     || TPDAS_A65M_adc2_fifo_paf_data()!=0
			   ) 
		  )
	       || TPDAS_A65M_adc1_fifo_ff_data()!=0 
	       || TPDAS_A65M_adc2_fifo_ff_data()!=0 
	      )
               goto error;

			// read and move FIFO data to two DSP on-chip memory areas with
			// 32-bit sign extension.
			// (only 1st _SRAM_DATA_SAMPLES FIFO samples 
			//  are moved to DSP on-chip RAM, since DSP on-chip RAM has
			//  smaller capacity than FIFO).
			// FIFO data data can veiwed via Code Composer STudio
			// graphic data viewer in time domain by setting pointers
			// to area 0x80008000 for ADC-1 and to 0x8000C000 for ADC-2.

	    		// ADC-1: verify that read-back ADC channel MUX is correct
	    fifo_data = TPDAS_A65M_adc1_fifo_rg_data();
            if (TPDAS_A65M_ADC_RIMUX_DATA(fifo_data)!= ADC_IMUX) goto error;
            if(i<_SRAM_DATA_SAMPLES) *mptr1++ = TPDAS_A65M_ADC_DATA(fifo_data); 

	    		// ADC-2: no RIMUX info, just save data
	    fifo_data = TPDAS_A65M_adc2_fifo_rg_data();
            if(i<_SRAM_DATA_SAMPLES) *mptr2++ = TPDAS_A65M_ADC_DATA(fifo_data);
	  }

			// ensure that EF/PAE are now both set to 'ON'
			// and PAF/FF is set to OFF
	if(   TPDAS_A65M_adc1_fifo_ef_data()==0
           || TPDAS_A65M_adc1_fifo_pae_data()==0
           || TPDAS_A65M_adc1_fifo_paf_data()!=0
           || TPDAS_A65M_adc1_fifo_ff_data()!=0
	   || TPDAS_A65M_adc2_fifo_ef_data()==0
           || TPDAS_A65M_adc2_fifo_pae_data()==0
           || TPDAS_A65M_adc2_fifo_paf_data()!=0
           || TPDAS_A65M_adc2_fifo_ff_data()!=0

          ) goto error;




    exit_ok:
    	while(1);	// get here on successful execution



    error:              // trap here on any error
    	while(1);	


}


