/********************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  OPM ADC-1/2 and DAC-1/2 test#1 for TORNADO-6x/P6x/E6x.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


   Description:
   ------------
	This demo performs DAQ test for ADC-1/2 and DAC-1/2 in the OPM DAQ mode
	as the following:
	  - on-board PFG 65MHz is used as sampling frequecy source
	  - DAC-1 FIFO is filled in with the SINE data. DAC-2 FIFO is filled in with
	    the rising ramp data. DAC-1/2 are loaded using DAC-IRT controller via PIOX IRQ-0
	    interrupt line.
	  - ADC A1/B1 analog inputs are used as input signals. Perform external connect
	    of DAC-1/2 outputs to ADC-A1/B1 inputs correspondingly.
	  - ADC/DAC s/w synchronization is used
	  - DAC FIFO PAF flag are used as DAQ TF
	  - DAC/ADC FIFO PAF flag are used as DAQ TF
	  - download ADC FIFO data to DSP on-chip memory using ADC-IRT controller 
	    cia PIOX IRQ-1 interrupt line, and check for correct number of words transmitted
	  - ADC-1/2 FIFO data are transferred into SBSRAM memory and 
	    can be veiwed via Code Composer signal viewer in time domain:
		- E6x/T6x:  
			- ADC-1 data are available at 0x00020200 address
		        - ADC-2 data are available at 0x00030200 address
		- TP6x:  
			- ADC-1 data are available at 0x02020200 address
		        - ADC-2 data are available at 0x02030200 address
	

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board. TI CCS tools are required to 
	recompile this demo.


  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/P6x mainboard
***********************************************************************/


#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#include	"math.h"
#include	"c6x_def.h"

	// ADC/DAC DAQ data length
#define	_DAQ_LEN		0x04000L


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#include			"te6x_dsp.h"
#define _REMOVE_PIOX_RESET	TE6X_set_pxsx_run(0x3)
#define	_PIOX16_SADDR_		TE6X_PIOX_SADDR

		// DAC/ADC data pointers
#define	_DAC1_DATA_PTR		(TE6X_SBSRAM_SADDR+0x200)
#define	_DAC2_DATA_PTR		(TE6X_SBSRAM_SADDR+(_DAQ_LEN*4L)+0x200)
#define	_ADC1_DATA_PTR		(TE6X_SBSRAM_SADDR+0x200+(_DAQ_LEN*4L*2L))
#define	_ADC2_DATA_PTR		(TE6X_SBSRAM_SADDR+(_DAQ_LEN*4L)+0x200+(_DAQ_LEN*4L*2L))
#endif

#if(_T6X_SELECTED)
#include			"t6x_dsp.h"
#define _REMOVE_PIOX_RESET	T6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		T6X_PIOX_SADDR

		// DAC/ADC data pointers
#define	_DAC1_DATA_PTR		(T6X_SBSRAM_SADDR+0x200)
#define	_DAC2_DATA_PTR		(T6X_SBSRAM_SADDR+(_DAQ_LEN*4L)+0x200)
#define	_ADC1_DATA_PTR		(T6X_SBSRAM_SADDR+0x200+(_DAQ_LEN*4L*2L))
#define	_ADC2_DATA_PTR		(T6X_SBSRAM_SADDR+(_DAQ_LEN*4L)+0x200+(_DAQ_LEN*4L*2L))
#endif

#if(_TP6X_SELECTED)
#include			"tp6x_dsp.h"
#define _REMOVE_PIOX_RESET	TP6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		TP6X_PIOX_SADDR

		// DAC/ADC data pointers
#define	_DAC1_DATA_PTR		(TP6X_SBSRAM_SADDR+0x200)
#define	_DAC2_DATA_PTR		(TP6X_SBSRAM_SADDR+(_DAQ_LEN*4L)+0x200)
#define	_ADC1_DATA_PTR		(TP6X_SBSRAM_SADDR+0x200+(_DAQ_LEN*4L*2L))
#define	_ADC2_DATA_PTR		(TP6X_SBSRAM_SADDR+(_DAQ_LEN*4L)+0x200+(_DAQ_LEN*4L*2L))
#endif
//--------------------------------------------------------------

// define host DSP platform for PA65_HD.H header file
#define _HOST_C6X_DSP_		1

//~~~~~~~~~~ include T/PDAS DCM header file ~~~~~~~~~~~~~~~~~~~~
#include	"pa65_hd.h"	
#include	"pa65x_hd.h"	
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



// perform host TORNADO specific EMIF initialization
init_cpu()
{
#if(_E6X_SELECTED)
	*C6X_EMIF_GCR=TE6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TE6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TE6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TE6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TE6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TE6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TE6X_EMIF_CE3_DATA;
#endif

#if(_T6X_SELECTED)
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif

#if(_TP6X_SELECTED)
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	// configure DSP INT0=PIOX_IRQ-0 and INT1=PIOX_IRQ-1
	TP6X_set_mirq_sel(1,TP6X_MIRQ_SEL_XIRQ3);
#endif

	_REMOVE_PIOX_RESET;			// release PIOX reset
	
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	// configure C6x on-chip IRQs #4/#5 to ext INT-0/1
	C6X_INTMUX_SET(5,C6X_INTSEL_XINT5);
}



	// set PFG to 65MHz = (25MHz*2*(V=130)/((R=50)*(S=2)*(X=1))
#define PFG_V_DATA		130
#define PFG_R_DATA      	50
#define PFG_S_DATA		TPDAS_A65M_S_DIV2
#define PFG_X_DATA		TPDAS_A65M_X_DIV1

	// define ADC input channel
#define	ADC_IMUX		TPDAS_A65M_ADC_IMUX_CH1


#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


main()
{ int i;
  int *mptr1, *mptr2;

	init_cpu();

	TPDAS_A65M_SET_HIRQ0_OFF;	// disable HIRQ-0 in order to retrigger on DAC IRT
	TPDAS_A65M_SET_HIRQ1_OFF;	// disable HIRQ-1 in order to retrigger on ADC IRT

		// configure PFG, Fs selector and ADC DAQ SYNC
	TPDAS_A65M_SET_FS_SEL_PFG;
	TPDAS_A65M_SET_ADC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_DAC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_SYNC_MODE_MASTER;

	TPDAS_A65M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	TPDAS_A65M_set_daq_mux_rg(ADC_IMUX);
	TPDAS_A65M_SET_DSIN_SEL_ADC;
	TPDAS_A65M_SET_DSOUT_SEL_DAC;
	TPDAS_A65M_SET_ADC_FMT_12;
	TPDAS_A65M_SET_DAC_FMT_12;
	TPDAS_A65M_CLR_ERR_ALL;			// clear all errors if they exist


        		// now reset ADC/DAC FIFO and program FIFO PAF/PAE flags
	if(   TPDAS_A65M_set_adc_fifo_offset(_DAQ_LEN/2,_DAQ_LEN)!=0
	   || TPDAS_A65M_set_dac_fifo_offset(_DAQ_LEN/2,_DAQ_LEN)!=0
	  )
            goto error;

	if(   TPDAS_A65M_adc1_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc1_fifo_pae_data() ==0
	   || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc1_fifo_paf_data() !=0
           || TPDAS_A65M_adc2_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc2_fifo_pae_data() ==0
	   || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc2_fifo_paf_data() !=0
	  )
            goto error;

	if(   TPDAS_A65M_dac1_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac1_fifo_pae_data() ==0
	   || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac1_fifo_paf_data() !=0
           || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac2_fifo_pae_data() ==0
	   || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac2_fifo_paf_data() !=0
	  )
            goto error;

	if(   TPDAS_A65M_adc_irt_err_data()   !=0
	   || TPDAS_A65M_dac_irt_err_data()   !=0       
	   || TPDAS_A65M_adc_fifo_ovf_data()  !=0
	   || TPDAS_A65M_dac_fifo_unf_data()  !=0
	   || TPDAS_A65M_adc_fifo_unf_data()  !=0
	   || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

			// set ADC FIFO DAQ mode with PAF termination flag and ADC-IRT EF flag
			// set DAC FIFO DAQ mode with EF termination flag and DAC-IRT PAF flag
			// and initialize DAQ procedure
	TPDAS_A65M_SET_ADC_MODE_OPM;
	TPDAS_A65M_SET_DAC_MODE_OPM;

	TPDAS_A65M_SET_ADC_DAQ_TF_SEL_PAF;
	TPDAS_A65M_SET_DAC_DAQ_TF_SEL_EF;
	TPDAS_A65M_SET_ADC_IRT_TF_SEL_EF;
	TPDAS_A65M_SET_DAC_IRT_TF_SEL_PAF;

			// perform ADC/DAC FIFO/DAQ reset
	TPDAS_A65M_RESET_DAQ;     

			// deselect source at PIOX IRQ-0/1 lines
	TPDAS_A65M_SET_HIRQ0_EN_ON;
	TPDAS_A65M_SET_HIRQ0_SEL_OFF;
	TPDAS_A65M_SET_HIRQ1_EN_ON;
	TPDAS_A65M_SET_HIRQ1_SEL_OFF;



			// now fill-in DAC-1/2 data arrays and transfer them to DAC-1/2 using DAC-IRT
	mptr1 = (int *) _DAC1_DATA_PTR;
	mptr2 = (int *) _DAC2_DATA_PTR;

	for(i=0;i<_DAQ_LEN;i++)
	  { 		//-- fill-in DAC-1 as SINE signal
		*mptr1++ = (int)((float)  (2000.0 * sin(6.28 *((float)(i%256))/256.0))) ;
		
			//-- fill-in DAC-2 as RAMP signal
		*mptr2++ = (int)((float)  (2000.0 * ((float)((i%256)-128))/128.0)) ;
	  }



/*  configure DMA-0/1 to transfer DAC-1/2 data via EXT_INT4 DSP interrupt (PIOX IRQ-0):
	- SRC DIR = +1, DST DIR = no
	- SRC SYNC = no, DST SYNC = EXT_INT4
	- esize = 32-bit words
	- PRI = DMA over CPU

   PCRG @ DMA-0:   00 00 1 0 0 1 00100 00000 0 0 00 00 00 01 00 00	
   PCRG @ DMA-1:   00 00 1 0 0 1 00100 00000 0 0 00 00 00 01 00 00	
	

*/
	*C6X_DMA0_PCRG=0x09200010;	
	*C6X_DMA0_SCRG=0;
	*C6X_DMA0_SARG=_DAC1_DATA_PTR;
	*C6X_DMA0_DARG=TPDAS_A65M_DAC1_FIFO_DATA_RG;
	*C6X_DMA0_TCRG=_DAQ_LEN;

	*C6X_DMA1_PCRG=0x09200010;	
	*C6X_DMA1_SCRG=0;
	*C6X_DMA1_SARG=_DAC2_DATA_PTR;
	*C6X_DMA1_DARG=TPDAS_A65M_DAC2_FIFO_DATA_RG;
	*C6X_DMA1_TCRG=_DAQ_LEN;

		// enable PIOX IRQ-0 line and select ADC-IRT irq sources
		// note!!!: 
		//	T6x/E6x require negative IRT signal,
		//      whereas TP6x require positive IRT    
#if(_TP6X_SELECTED)
	TPDAS_A65M_SET_HIRQ0_DAC_IRT_POS;
#else
	TPDAS_A65M_SET_HIRQ0_DAC_IRT_NEG;
#endif

	*C6X_DMA0_PCRG|=0x1; // start DMA0
	*C6X_DMA1_PCRG|=0x1; // start DMA1

		// wait until both DMA-0/1 stops or DAC IRT error occurs
	while(   ((*C6X_DMA0_PCRG)&0xc)!=0
	      || ((*C6X_DMA1_PCRG)&0xc)!=0
	     )
	  { if(TPDAS_A65M_dac_irt_err_data()!=0)  
		goto error;
	  }

	if(    TPDAS_A65M_adc_irt_err_data()   !=0
	    || TPDAS_A65M_dac_irt_err_data()   !=0
	    || TPDAS_A65M_adc_fifo_ovf_data()  !=0
	    || TPDAS_A65M_adc_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

	if(    TPDAS_A65M_dac1_fifo_ef_data()  !=0
	    || TPDAS_A65M_dac1_fifo_pae_data() !=0
	    || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac1_fifo_paf_data() ==0
            || TPDAS_A65M_dac2_fifo_ef_data()  !=0
	    || TPDAS_A65M_dac2_fifo_pae_data() !=0
	    || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac2_fifo_paf_data() ==0
	  )
            goto error;

	if(    TPDAS_A65M_adc1_fifo_ef_data()  ==0
	    || TPDAS_A65M_adc1_fifo_pae_data() ==0
	    || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc1_fifo_paf_data() !=0
            || TPDAS_A65M_adc2_fifo_ef_data()  ==0
	    || TPDAS_A65M_adc2_fifo_pae_data() ==0
	    || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc2_fifo_paf_data() !=0
	  )
            goto error;


			// now initialize ADC/DAC DAQ processes
	TPDAS_A65M_set_daq_cntr2_rg(TPDAS_A65M_ADC_DAQ_RUN | TPDAS_A65M_DAC_DAQ_RUN);
	if(   TPDAS_A65M_adc_daq_run_data() == 0
	   || TPDAS_A65M_adc_daq_end_data() != 0
	   || TPDAS_A65M_dac_daq_run_data() == 0
	   || TPDAS_A65M_dac_daq_end_data() != 0
	  ) goto error;

			// wait until ADC/DAC DAQ terminates
	while(    TPDAS_A65M_adc_daq_end_data()==0
	       || TPDAS_A65M_dac_daq_end_data()==0
	     );


	if(    TPDAS_A65M_adc_irt_err_data()   !=0
	    || TPDAS_A65M_dac_irt_err_data()   !=0
	    || TPDAS_A65M_adc_fifo_ovf_data()  !=0
	    || TPDAS_A65M_adc_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

	if(    TPDAS_A65M_dac1_fifo_ef_data()  ==0
	    || TPDAS_A65M_dac1_fifo_pae_data() ==0
	    || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac1_fifo_paf_data() !=0
            || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	    || TPDAS_A65M_dac2_fifo_pae_data() ==0
	    || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac2_fifo_paf_data() !=0
	  )
            goto error;

	if(    TPDAS_A65M_adc1_fifo_ef_data()  !=0
	    || TPDAS_A65M_adc1_fifo_pae_data() !=0
	    || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc1_fifo_paf_data() ==0
            || TPDAS_A65M_adc2_fifo_ef_data()  !=0
	    || TPDAS_A65M_adc2_fifo_pae_data() !=0
	    || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc2_fifo_paf_data() ==0
	  )
            goto error;





		// now download ADC FIFO data using ADC-IRT and store it in SBSRAM,
		// ADC FIFO data can be veiwed via Code Composer STudio
		// graphic data viewer in time domain by setting pointers
		// to the corresponding SBSRAM area (_ADC1_DATA_PTR and _ADC2_DATA_PTR)


/*  configure DMA-0/1 to transfer ADC-1/2 data via EXT_INT5 DSP interrupt (PIOX IRQ-1):
	- SRC DIR = 0, DST DIR = +1
	- SRC SYNC = EXT_INT5, DST SYNC = no
	- esize = 32-bit words
	- PRI = DMA over CPU

   PCRG @ DMA-0:   00 00 1 0 0 1 00000 00101 0 0 00 00 01 00 00 00	
   PCRG @ DMA-1:   00 00 1 0 0 1 00000 00101 0 0 00 00 01 00 00 00	
	

*/

	*C6X_DMA0_PCRG=0x09014040;	
	*C6X_DMA0_SCRG=0;
	*C6X_DMA0_SARG=TPDAS_A65M_ADC1_FIFO_DATA_RG;
	*C6X_DMA0_DARG=_ADC1_DATA_PTR;
	*C6X_DMA0_TCRG=_DAQ_LEN;

	*C6X_DMA1_PCRG=0x09014040;	
	*C6X_DMA1_SCRG=0;
	*C6X_DMA1_SARG=TPDAS_A65M_ADC2_FIFO_DATA_RG;
	*C6X_DMA1_DARG=_ADC2_DATA_PTR;
	*C6X_DMA1_TCRG=_DAQ_LEN;

		// enable PIOX IRQ-1 line and select ADC-IRT irq sources
		// note!!!: 
		//	T6x/E6x require negative ADC IRT signal,
		//      whereas TP6x require positive ADC IRT    
#if(_TP6X_SELECTED)
	TPDAS_A65M_SET_HIRQ1_ADC_IRT_POS;
#else
	TPDAS_A65M_SET_HIRQ1_ADC_IRT_NEG;
#endif

	*C6X_DMA0_PCRG|=0x1; // start DMA0
	*C6X_DMA1_PCRG|=0x1; // start DMA1

		// wait until both DMA-0/1 stops or ADC IRT error occurs
	while(   ((*C6X_DMA0_PCRG)&0xc)!=0
	      || ((*C6X_DMA1_PCRG)&0xc)!=0
	     )
	  { if(TPDAS_A65M_adc_irt_err_data()!=0) goto error;
	  }

	if(    TPDAS_A65M_adc_irt_err_data()   !=0
	    || TPDAS_A65M_dac_irt_err_data()   !=0
	    || TPDAS_A65M_adc_fifo_ovf_data()  !=0
	    || TPDAS_A65M_adc_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

	if(    TPDAS_A65M_dac1_fifo_ef_data()  ==0
	    || TPDAS_A65M_dac1_fifo_pae_data() ==0
	    || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac1_fifo_paf_data() !=0
            || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	    || TPDAS_A65M_dac2_fifo_pae_data() ==0
	    || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac2_fifo_paf_data() !=0
            || TPDAS_A65M_adc1_fifo_ef_data()  ==0
	    || TPDAS_A65M_adc1_fifo_pae_data() ==0
	    || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc1_fifo_paf_data() !=0
            || TPDAS_A65M_adc2_fifo_ef_data()  ==0
	    || TPDAS_A65M_adc2_fifo_pae_data() ==0
	    || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc2_fifo_paf_data() !=0
	  )
            goto error;

	  		// now analyze returned ADC_RIMUX data in ADC-1 data words,
			// and format ADC-1/2 data to 32-bit signed data
	mptr1 = (int *) _ADC1_DATA_PTR;
	mptr2 = (int *) _ADC2_DATA_PTR;

	for(i=0; i<_DAQ_LEN; i++,mptr1++,mptr2++)
	  {
	    if (TPDAS_A65M_ADC_RIMUX_DATA(*mptr1)!= ADC_IMUX) goto error;
            *mptr1 = TPDAS_A65M_ADC_DATA(*mptr1); 
            *mptr2 = TPDAS_A65M_ADC_DATA(*mptr2); 
	  }


    exit_ok:
    	while(1);	// get here on successful execution



    error:              	// trap here on any error
	*C6X_DMA0_PCRG=0; 	// stop DMA0
	*C6X_DMA1_PCRG=0; 	// stop DMA1
        TPDAS_A65M_RESET_DAQ;	// reset DAQ/IRT controllers
    	while(1);	


}


