/********************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  PTM ADC/DAC-1/2 test for TORNADO-6x/P6x/E6x.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


   Description:
   ------------
	This demo performs path-through test for ADC-1/2 and DAC-1/2 in the PTM DAQ mode
	as the following:
	  - on-board PFG 1.5MHz is used as sampling frequecy source
	  - ADC channels A0/B0 are used for analog input
	  - ADC-1 data (AIN-A0 input) is continuously routed to DAC-1 output, 
	    and ADC-2 data (AIN-B0 input) is continuously routed to DAC-2 output.
	  - DAC s/w synchronization is used
	  - DAC/ADC IRT modes are used to transfer ADC data to DAC in real-time:
		- PIOX IRQ-0 is used for DAC IRT via C6x DSP on-chip DMA-0/1
		- PIOX IRQ-1 is used for ADC IRT via C6x DSP on-chip DMA-2/3
	  	- DAC FIFO PAE flag are used as DAQ TF, and PAF flag is used 
		  as DAC-IRT flag
	  	- ADC FIFO PAF flag are used as DAQ TF, and PAE flag is used 
		  as ADC-IRT flag
		- C6x DSP on-chip data memory is used for AD/DA circular data storage

   IMPORTANT NOTE !!!
   ------------------
	This demo runs at 1.5MHz sampling frequency clock, which is below the
	minimum sampling frequency specs (6.5MHz) for on-board ADC (refer to User's
	Guide for detail technical spec). Therefore, it is not guaranteed that the
	DAC-1/2 outputs will appear as non-distorted AIN-A0/B0 analog inputs. However
	due to results of testing multiple T/PDAS-AD8/12D/65M-DA2/12D/65M DCM, it must work
	properly, since actual break of ADC chip operation has been found to appear at
	the sampling frequencies 0.75MHz and below. Anyway, this demo is provided 
	for refernce purposes only in order to demonstrate how to program pass-through
	ADC/DAC data transfer using C6x DSP on-chip DMA-0/1/2/3 controllers.
	
	
	
   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board. TI CCS tools are required to 
	recompile this demo.


  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/P6x mainboard
***********************************************************************/


#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#include	"math.h"
#include	"c6x_def.h"


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#include			"te6x_dsp.h"
#define _REMOVE_PIOX_RESET	TE6X_set_pxsx_run(0x3)
#define	_PIOX16_SADDR_		TE6X_PIOX_SADDR
#endif

#if(_T6X_SELECTED)
#include			"t6x_dsp.h"
#define _REMOVE_PIOX_RESET	T6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		T6X_PIOX_SADDR
#endif

#if(_TP6X_SELECTED)
#include			"tp6x_dsp.h"
#define _REMOVE_PIOX_RESET	TP6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		TP6X_PIOX_SADDR
#endif
//--------------------------------------------------------------

// define host DSP platform for PA65_HD.H header file
#define _HOST_C6X_DSP_		1

//~~~~~~~~~~ include T/PDAS DCM header file ~~~~~~~~~~~~~~~~~~~~
#include	"pa65_hd.h"	
#include	"pa65x_hd.h"	
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



// perform host TORNADO specific EMIF initialization
init_cpu()
{
#if(_E6X_SELECTED)
	*C6X_EMIF_GCR=TE6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TE6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TE6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TE6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TE6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TE6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TE6X_EMIF_CE3_DATA;
#endif

#if(_T6X_SELECTED)
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif

#if(_TP6X_SELECTED)
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	// configure DSP INT0=PIOX_IRQ-0 and INT1=PIOX_IRQ-1
	TP6X_set_mirq_sel(1,TP6X_MIRQ_SEL_XIRQ3);
#endif

	_REMOVE_PIOX_RESET;			// release PIOX reset
	
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	// configure C6x on-chip IRQs #4/#5 to ext INT-0/1
	C6X_INTMUX_SET(5,C6X_INTSEL_XINT5);
}



	// set PFG to 1.5MHz = (25MHz*2*(V=120)/((R=50)*(S=5)*(X=16))
#define PFG_V_DATA		120
#define PFG_R_DATA      	50
#define PFG_S_DATA		TPDAS_A65M_S_DIV5
#define PFG_X_DATA		TPDAS_A65M_X_DIV16

	// define ADC input channel
#define	ADC_IMUX		TPDAS_A65M_ADC_IMUX_CH0


#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency

	// ADC/DAC DAQ data length and DAC/ADC data pointers
#define	_DAQ_LEN		0x01000L
#define	_ADDA1_DATA_PTR		(0x80008000)
#define	_ADDA2_DATA_PTR		(0x80008000+(_DAQ_LEN*4L))


main()
{ int i;
  int *mptr1, *mptr2;

	init_cpu();

	TPDAS_A65M_SET_HIRQ0_OFF;	// disable HIRQ-0 in order to retrigger on DAC IRT
	TPDAS_A65M_SET_HIRQ1_OFF;	// disable HIRQ-1 in order to retrigger on ADC IRT

	TPDAS_A65M_RESET_DAQ;		// stop and reset DAQ/FIFO controllers

		// configure PFG, Fs selector and ADC DAQ SYNC
	TPDAS_A65M_SET_FS_SEL_PFG;
	TPDAS_A65M_SET_ADC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_DAC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_SYNC_MODE_MASTER;

	TPDAS_A65M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	TPDAS_A65M_set_daq_mux_rg(ADC_IMUX);
	TPDAS_A65M_SET_DSIN_SEL_ADC;
	TPDAS_A65M_SET_DSOUT_SEL_DAC;
	TPDAS_A65M_SET_ADC_FMT_12;
	TPDAS_A65M_SET_DAC_FMT_12;
	TPDAS_A65M_CLR_ERR_ALL;			// clear all errors if they exist


        		// now reset DAC FIFO and program FIFO PAF/PAE flags
	if(   TPDAS_A65M_set_adc_fifo_offset(_DAQ_LEN/2,_DAQ_LEN)!=0
           || TPDAS_A65M_set_dac_fifo_offset(_DAQ_LEN/2,_DAQ_LEN)!=0
	  )
            goto error;

	if(   TPDAS_A65M_adc1_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc1_fifo_pae_data() ==0
	   || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc1_fifo_paf_data() !=0
           || TPDAS_A65M_adc2_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc2_fifo_pae_data() ==0
	   || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc2_fifo_paf_data() !=0
	  )
            goto error;

	if(   TPDAS_A65M_dac1_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac1_fifo_pae_data() ==0
	   || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac1_fifo_paf_data() !=0
           || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac2_fifo_pae_data() ==0
	   || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac2_fifo_paf_data() !=0
	  )
            goto error;

	if(   TPDAS_A65M_adc_irt_err_data()   !=0
	   || TPDAS_A65M_dac_irt_err_data()   !=0       
	   || TPDAS_A65M_adc_fifo_ovf_data()  !=0
	   || TPDAS_A65M_dac_fifo_unf_data()  !=0
	   || TPDAS_A65M_adc_fifo_unf_data()  !=0
	   || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

			// set DAC FIFO DAQ mode with PAE termination flag and DAC-IRT PAF flag
			// and initialize DAQ procedure
	TPDAS_A65M_SET_ADC_MODE_PTM;
	TPDAS_A65M_SET_DAC_MODE_PTM;

	TPDAS_A65M_SET_ADC_DAQ_TF_SEL_PAF;
	TPDAS_A65M_SET_ADC_IRT_TF_SEL_PAE;
	TPDAS_A65M_SET_DAC_DAQ_TF_SEL_PAE;
	TPDAS_A65M_SET_DAC_IRT_TF_SEL_PAF;

			// perform ADC/DAC FIFO/DAQ reset
	TPDAS_A65M_RESET_DAQ;     

			// deselect source at PIOX IRQ-0/1 lines
	TPDAS_A65M_SET_HIRQ0_EN_ON;
	TPDAS_A65M_SET_HIRQ0_SEL_OFF;
	TPDAS_A65M_SET_HIRQ1_EN_ON;
	TPDAS_A65M_SET_HIRQ1_SEL_OFF;

			// clear first-pass DAC data 
	mptr1 = (int *) _ADDA1_DATA_PTR;
	mptr2 = (int *) _ADDA2_DATA_PTR;

	for(i=0;i<_DAQ_LEN;i++)
	  { 	*mptr1++ = 0;
		*mptr2++ = 0;
	  }

/*  configure DMA-0/1 to transfer DAC-1/2 data via EXT_INT4 DSP interrupt (PIOX IRQ-0)
    in auto-init mode in oder to exclude successive reloads of DMA configuration during
    continuous DAC PTM mode:
	- SRC DIR = +1, DST DIR = no
	- SRC SYNC = no, DST SYNC = EXT_INT4
	- esize = 32-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = from global address B/C
	- DST RELOAD = NO 
	- CNT RELOAD = from global counter rg A

   PCRG @ DMA-0:   00 01 1 0 0 1 00100 00000 0 0 00 00 00 01 00 00	
   PCRG @ DMA-1:   00 10 1 0 0 1 00100 00000 0 0 00 00 00 01 00 00	


    configure DMA-0/1 to transfer ADC-1/2 data via EXT_INT5 DSP interrupt (PIOX IRQ-1)
    in auto-init mode in oder to exclude successive reloads of DMA configuration during
    continuous DAC PTM mode:
	- SRC DIR = no, DST DIR = +1
	- SRC SYNC = EXT_INT5, DST SYNC = no
	- esize = 32-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = NO 
	- DST RELOAD = from global address B/C
	- CNT RELOAD = from global counter rg A

   PCRG @ DMA-2:   01 00 1 1 0 1 00000 00101 0 0 00 00 01 00 00 00	
   PCRG @ DMA-3:   10 00 1 1 0 1 00000 00101 0 0 00 00 01 00 00 00	


*/

	*C6X_DMA0_PCRG=0x19200010;	
	*C6X_DMA0_SCRG=0;
	*C6X_DMA0_SARG=_ADDA1_DATA_PTR;
	*C6X_DMA0_DARG=TPDAS_A65M_DAC1_FIFO_DATA_RG;
	*C6X_DMA0_TCRG=_DAQ_LEN;

	*C6X_DMA1_PCRG=0x29200010;	
	*C6X_DMA1_SCRG=0;
	*C6X_DMA1_SARG=_ADDA2_DATA_PTR;
	*C6X_DMA1_DARG=TPDAS_A65M_DAC2_FIFO_DATA_RG;
	*C6X_DMA1_TCRG=_DAQ_LEN;

	*C6X_DMA2_PCRG=0x4b014040;	
	*C6X_DMA2_SARG=TPDAS_A65M_ADC1_FIFO_DATA_RG;
	*C6X_DMA2_DARG=_ADDA1_DATA_PTR;
	*C6X_DMA2_TCRG=_DAQ_LEN;

	*C6X_DMA3_PCRG=0x8b014040;	
	*C6X_DMA3_SARG=TPDAS_A65M_ADC2_FIFO_DATA_RG;
	*C6X_DMA3_DARG=_ADDA2_DATA_PTR;
	*C6X_DMA3_TCRG=_DAQ_LEN;


		// global DMA register for auto-init reload:
	*C6X_DMA0_GCRARG= _DAQ_LEN;		// counter reload (DMA-0/1/2/3)
	*C6X_DMA0_GABRG = _ADDA1_DATA_PTR;	// source address DMA-0/2
	*C6X_DMA0_GACRG = _ADDA2_DATA_PTR;	// source address DMA-1/3


		// enable PIOX IRQ-0/1 line and select DAC/ADC-IRT irq sources
		// note!!!: 
		//	T6x/E6x require negative DAC/ADC IRT signal,
		//      whereas TP6x require positive DAC/ADC IRT    
#if(_TP6X_SELECTED)
	TPDAS_A65M_SET_HIRQ0_DAC_IRT_POS;
	TPDAS_A65M_SET_HIRQ1_ADC_IRT_POS;
#else
	TPDAS_A65M_SET_HIRQ0_DAC_IRT_NEG;
	TPDAS_A65M_SET_HIRQ1_ADC_IRT_NEG;
#endif

	*C6X_DMA2_SCRG=0xA000;		// clear false ADC-1 DMA-2 RSYNC event
	*C6X_DMA3_SCRG=0xA000;		// clear false ADC-2 DMA-3 RSYNC event

	*C6X_DMA0_PCRG|=0x3; // start DMA0 in auto-init mode
	*C6X_DMA1_PCRG|=0x3; // start DMA1 in auto-init mode
	*C6X_DMA2_PCRG|=0x3; // start DMA2 in auto-init mode
	*C6X_DMA3_PCRG|=0x3; // start DMA3 in auto-init mode

		// wait until DAC1-FIFO_PAF and DAC2-FIFO_PAF are set in DAC-IRT. 
		// Check for errors and exit on any error.
	while(TPDAS_A65M_dac1_fifo_paf_data()==0 || TPDAS_A65M_dac2_fifo_paf_data()==0)
         { if(TPDAS_A65M_err_stat_rg_data()!=0) goto error;
	 }    

		// Enable ADCC-DAQ, then wait a little, and enable DAC-DAQ.
		// Short pause between ADC-DAQ start and DAC-DAQ start will allow the ADC-IRT 
		// to run slightly ahead of DAC-IRT and to guarantee that DAC output data
		// contains consistent ADC data, which has been already recorded in SRAM.
	TPDAS_A65M_SET_ADC_DAQ_RUN;
	for(i=0;i<20;i++);
	TPDAS_A65M_SET_DAC_DAQ_RUN;

	if(   TPDAS_A65M_dac_daq_run_data() == 0
	   || TPDAS_A65M_adc_daq_run_data() == 0
	  ) goto error;
	

		// loop here infinitely with DMA auto-init
		// check for errors during DAQ. Abort on any error.
	while(1)
         { if(TPDAS_A65M_err_stat_rg_data()!=0) goto error;
	   for(i=0;i<10000;i++);		// wait prior next error check in order to reduce bus load
	   
	 }

    error:              // trap here on any error
	*C6X_DMA0_PCRG=0;	// stop DMA0
        *C6X_DMA1_PCRG=0;	// stop DMA1
	*C6X_DMA2_PCRG=0;	// stop DMA2
        *C6X_DMA3_PCRG=0; 	// stop DMA3
        TPDAS_A65M_RESET_DAQ;	// reset DAQ/FIFO
    	while(1);	


}


