/********************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  PTM DAC-1/2 test using software data upload for TORNADO-6x/P6x/E6x.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


   Description:
   ------------
	This demo performs test for DAC-1/2 in the PTM DAQ mode
	as the following:
	  - on-board PFG 125kHz is used as sampling frequecy source
	  - DAC-1 data array is filled in with the SINE data. DAC-2 data array is filled in 
	    with the rising ramp data. 
	  - DAC-1/2 are uploaded using via host DSP software without IRT and DMA. Two
	    software upload methods are provided, which can be selected by the user
	    by means of uncommenting the corresponding code (see below).
	  - DAC s/w synchronization is used
	

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board. TI CCS tools are required to 
	recompile this demo.


  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/P6x mainboard
***********************************************************************/


#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#include	"math.h"
#include	"c6x_def.h"


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#include			"te6x_dsp.h"
#define _REMOVE_PIOX_RESET	TE6X_set_pxsx_run(0x3)
#define	_PIOX16_SADDR_		TE6X_PIOX_SADDR
#endif

#if(_T6X_SELECTED)
#include			"t6x_dsp.h"
#define _REMOVE_PIOX_RESET	T6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		T6X_PIOX_SADDR
#endif

#if(_TP6X_SELECTED)
#include			"tp6x_dsp.h"
#define _REMOVE_PIOX_RESET	TP6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		TP6X_PIOX_SADDR
#endif
//--------------------------------------------------------------

// define host DSP platform for PA65_HD.H header file
#define _HOST_C6X_DSP_		1

//~~~~~~~~~~ include T/PDAS DCM header file ~~~~~~~~~~~~~~~~~~~~
#include	"pa65_hd.h"	
#include	"pa65x_hd.h"
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



// perform host TORNADO specific EMIF initialization
init_cpu()
{
#if(_E6X_SELECTED)
	*C6X_EMIF_GCR=TE6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TE6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TE6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TE6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TE6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TE6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TE6X_EMIF_CE3_DATA;
#endif

#if(_T6X_SELECTED)
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif

#if(_TP6X_SELECTED)
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);
#endif

	_REMOVE_PIOX_RESET;			// release PIOX reset
}



	// set PFG to 125kHz = (25MHz*2*(V=20)/((R=50)*(S=10)*(X=16))
#define PFG_V_DATA		20
#define PFG_R_DATA      	50
#define PFG_S_DATA		TPDAS_A65M_S_DIV10
#define PFG_X_DATA		TPDAS_A65M_X_DIV16

	// define ADC input channel
#define	ADC_IMUX		TPDAS_A65M_ADC_IMUX_CH1


#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency

	// ADC/DAC DAQ data length
#define	_DAQ_LEN		0x01000

		// DAC-1/2 data array pointers
#define	_DAC1_DATA_PTR		(0x80008000)	
#define	_DAC2_DATA_PTR		(0x80008000+(_DAQ_LEN*4))	


#define	_dac_pae_offset		(_DAQ_LEN/2)
#define	_dac_paf_offset		_DAQ_LEN


main()
{ int i;
  int *mptr1, *mptr2, *dac1, *dac2, dac1_cnt, dac2_cnt, dac1_ok, dac2_ok;

	init_cpu();


		// configure PFG, Fs selector and DAC DAQ SYNC
	TPDAS_A65M_SET_FS_SEL_PFG;
	TPDAS_A65M_SET_DAC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_SYNC_MODE_MASTER;

	TPDAS_A65M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	TPDAS_A65M_set_daq_mux_rg(ADC_IMUX);
	TPDAS_A65M_SET_DSOUT_SEL_DAC;
	TPDAS_A65M_SET_DAC_FMT_12;


        		// now reset DAC FIFO and program FIFO PAF/PAE flags
	if(   TPDAS_A65M_set_dac_fifo_offset(_dac_pae_offset,_dac_paf_offset)!=0
	  )
            goto error;

			// set DAC PTM DAQ mode
	TPDAS_A65M_SET_DAC_MODE_PTM;
	TPDAS_A65M_RESET_DAC_DAQ;     		// perform DAC FIFO/DAQ reset
	TPDAS_A65M_CLR_ERR_ALL;			// clear all errors if they exist

	if(   TPDAS_A65M_dac1_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac1_fifo_pae_data() ==0
	   || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac1_fifo_paf_data() !=0
           || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac2_fifo_pae_data() ==0
	   || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac2_fifo_paf_data() !=0
	   || TPDAS_A65M_dac_irt_err_data()   !=0       
	   || TPDAS_A65M_dac_fifo_unf_data()  !=0
	   || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

			// now fill-in DAC-1/2 data arrays and transfer them to DAC-1/2 using DAC-IRT
	mptr1 = (int *) _DAC1_DATA_PTR;
	mptr2 = (int *) _DAC2_DATA_PTR;

	for(i=0;i<_DAQ_LEN;i++)
	  { 		//-- fill-in DAC-1 as SINE signal
		*mptr1++ = (int)((float)  (2000.0 * sin(6.28 *((float)(i%256))/256.0))) ;
		
			//-- fill-in DAC-2 as RAMP signal
		*mptr2++ = (int)((float)  (2000.0 * ((float)((i%256)-128))/128.0)) ;
	  }


//------ inserts for s/w data upload

	dac1  = (int *) TPDAS_A65M_DAC1_FIFO_DATA_RG;
	dac2  = (int *) TPDAS_A65M_DAC2_FIFO_DATA_RG;
	mptr1 = (int *) _DAC1_DATA_PTR;
	mptr2 = (int *) _DAC2_DATA_PTR;

	for(i=0;i<_DAQ_LEN;i++)
	{
	  *dac1 = *mptr1++;
	  *dac2 = *mptr2++;

			// verify DAC FIFO flag procurement
			// this test requires high enough DAC FIFO RCLK in order EF/PAE flags be able to update
			// uncomment the PAE test operators in case of fast RCLK
	  if( /*   (i>0 &&  TPDAS_A65M_dac1_fifo_ef_data()!=0)
             ||  ((i+1)<=_dac_pae_offset && TPDAS_A65M_dac1_fifo_pae_data()==0)
             ||  ((i+1)>_dac_pae_offset  && TPDAS_A65M_dac1_fifo_pae_data()!=0)
             ||*/  ((i+1)<_dac_paf_offset  && TPDAS_A65M_dac1_fifo_paf_data()!=0)
             ||  ((i+1)>=_dac_paf_offset && TPDAS_A65M_dac1_fifo_paf_data()==0)
             ||  TPDAS_A65M_dac1_fifo_ff_data()!=0
	    ) 
		goto error;

	  if( /*   (i>0 &&  TPDAS_A65M_dac2_fifo_ef_data()!=0)
             ||  ((i+1)<=_dac_pae_offset && TPDAS_A65M_dac2_fifo_pae_data()==0)
             ||  ((i+1)>_dac_pae_offset  && TPDAS_A65M_dac2_fifo_pae_data()!=0)
             ||*/  ((i+1)<_dac_paf_offset  && TPDAS_A65M_dac2_fifo_paf_data()!=0)
             ||  ((i+1)>=_dac_paf_offset && TPDAS_A65M_dac2_fifo_paf_data()==0)
             ||  TPDAS_A65M_dac2_fifo_ff_data()!=0
	    ) 
		goto error;
	}

		// check for initial load is OK
        if(      TPDAS_A65M_dac1_fifo_paf_data()==0
	      || TPDAS_A65M_dac2_fifo_paf_data()==0
	  )
                 goto error;

	mptr1 = (int *) _DAC1_DATA_PTR;
	mptr2 = (int *) _DAC2_DATA_PTR;
	dac1_cnt = 0;
	dac2_cnt = 0;
	dac1_ok=1;
	dac2_ok=1;


		// configure GPIO-0/1 as outputs:
		//  GPIO-0 will indicate active DAC1-FIFO upload 
		//  GPIO-1 will indicate active DAC2-FIFO upload
	TPDAS_A65M_SET_GPIO0_DIR_OUTPUT;
	TPDAS_A65M_SET_GPIO1_DIR_OUTPUT;	
	TPDAS_A65M_SET_GPIO0_DATA_OFF;
	TPDAS_A65M_SET_GPIO1_DATA_OFF;

		// now enable DAC-DAQ
	TPDAS_A65M_SET_DAC_DAQ_RUN;
	if(   TPDAS_A65M_dac_daq_run_data() == 0
	   || TPDAS_A65M_dac_daq_end_data() != 0
	  ) goto error;
	


		// loop here infinitely DAC FIFO s/w upload
		// check for errors during DAQ. Abort on any error.
	while(1)
         {

/*-------- continuous DAC FIFO upload procedure using DAC FIFO PAF flags ------------
 This upload method always keeps DAC FIFO almost full,
 and in case FIFO goes down below the PAF flag indicator, it
 will be uploaded to the almost full condition again.
 This method requires a lot of DSP performance to track the PAF condition.
*/
/*			// check for DAC FIFO DAQ errors
	        if (    TPDAS_A65M_dac_fifo_unf_data()!=0
	             || TPDAS_A65M_dac_fifo_ovf_data()!=0
		   )
			goto error;


 			// check for DAC1 FIFO PAE flag comes active
		if(TPDAS_A65M_dac1_fifo_paf_data()==0)
 		  {  
			  TPDAS_A65M_SET_GPIO0_DATA_ON;
			  *dac1 = *mptr1++;
			  if ((++dac1_cnt)>=_DAQ_LEN) 
				{ dac1_cnt=0;
				  mptr1 = (int *)_DAC1_DATA_PTR;
				}
			  if(TPDAS_A65M_dac1_fifo_paf_data()!=0) TPDAS_A65M_SET_GPIO0_DATA_OFF;
		  }


 			// check for DAC2 FIFO PAE flag comes active
		if(TPDAS_A65M_dac2_fifo_paf_data()==0)
 		  {  
			  TPDAS_A65M_SET_GPIO1_DATA_ON;
			  *dac2 = *mptr2++;
			  if ((++dac2_cnt)>=_DAQ_LEN) 
				{ dac2_cnt=0;
				  mptr2 = (int *)_DAC2_DATA_PTR;
				}
			  if(TPDAS_A65M_dac2_fifo_paf_data()!=0) TPDAS_A65M_SET_GPIO1_DATA_OFF;
		  }
*/
/*-----------------------------------------------------------------------------------*/


/*-------- continuous DAC FIFO upload procedure using DAC FIFO PAE/PAF flags ------------
 This upload method is awaiting for the DAC FIFO PAE flag coming active, and
 then uploads data to DAC FIFO until DAC PAF flag will come active.
  
 This method is the software simulation of the DAC IRT procedure with 
 the PAE/PAF termination flags.

 This method requires less of DSP performance, since DSP can be activated only
 when PAE flag comes active.
*/
		
			// check for DAC FIFO DAQ errors
		if (    TPDAS_A65M_dac_fifo_unf_data()!=0
		     || TPDAS_A65M_dac_fifo_ovf_data()!=0
		   )
		     goto error;

	 		// wait for DAC1/2 FIFO PAE flags come active
			// and set upload enable flags
		if(TPDAS_A65M_dac1_fifo_pae_data()!=0) 
			{ 
				dac1_ok=0;
				TPDAS_A65M_SET_GPIO0_DATA_ON;
			}

		if(TPDAS_A65M_dac2_fifo_pae_data()!=0) 
			{ 
				dac2_ok=0;
				TPDAS_A65M_SET_GPIO1_DATA_ON;
			}
	

			// fill-in DAC1 and DAC2 FIFOs until PAF flags come active
                if(dac1_ok==0)
			{ 
			  *dac1 = *mptr1++;
			  if ((++dac1_cnt)>=_DAQ_LEN) 
				{
				  dac1_cnt=0;
				  mptr1 = (int *)_DAC1_DATA_PTR;
				}
			  if(TPDAS_A65M_dac1_fifo_paf_data()!=0) 
				{ 
 					dac1_ok=1;
					TPDAS_A65M_SET_GPIO0_DATA_OFF;
				}
			}


                if(dac2_ok==0)
			{ 
			  *dac2 = *mptr2++;
			  if ((++dac2_cnt)>=_DAQ_LEN) 
				{
				  dac2_cnt=0;
				  mptr2 = (int *)_DAC2_DATA_PTR;
				}
			  if(TPDAS_A65M_dac2_fifo_paf_data()!=0) 
				{ 
 					dac2_ok=1;
					TPDAS_A65M_SET_GPIO1_DATA_OFF;
				}
			}
	  }

//------------------------------------------



    error:              // trap here on any error
	TPDAS_A65M_SET_DAC_DAQ_ABORT;
    	while(1);	


}


