/********************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  PTM DAC-1/2 test using DAC IRT controller for TORNADO-6x/P6x/E6x.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


   Description:
   ------------
	This demo performs test for DAC-1/2 in the PTM DAQ mode
	as the following:
	  - on-board 1MHz PFG is used as sampling frequecy source
	  - DAC-1 data array is filled in with the SINE data. DAC-2 data array is filled in 
	    with the rising ramp data. 
	  - DAC-1/2 are uploaded using DAC IRT controller via host PIOX-16 IRQ-0/1 and 'C6x DMA-0/1
	  - DAC s/w synchronization is used
	

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board. TI CCS tools are required to 
	recompile this demo.


  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/P6x mainboard
***********************************************************************/


#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#include	"math.h"
#include	"c6x_def.h"


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#include			"te6x_dsp.h"
#define _REMOVE_PIOX_RESET	TE6X_set_pxsx_run(0x3)
#define	_PIOX16_SADDR_		TE6X_PIOX_SADDR
#endif

#if(_T6X_SELECTED)
#include			"t6x_dsp.h"
#define _REMOVE_PIOX_RESET	T6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		T6X_PIOX_SADDR
#endif

#if(_TP6X_SELECTED)
#include			"tp6x_dsp.h"
#define _REMOVE_PIOX_RESET	TP6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		TP6X_PIOX_SADDR
#endif
//--------------------------------------------------------------

// define host DSP platform for PA65_HD.H header file
#define _HOST_C6X_DSP_		1

//~~~~~~~~~~ include T/PDAS DCM header file ~~~~~~~~~~~~~~~~~~~~
#include	"pa65_hd.h"	
#include	"pa65x_hd.h"
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



// perform host TORNADO specific EMIF initialization
init_cpu()
{
#if(_E6X_SELECTED)
	*C6X_EMIF_GCR=TE6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TE6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TE6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TE6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TE6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TE6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TE6X_EMIF_CE3_DATA;
#endif

#if(_T6X_SELECTED)
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif

#if(_TP6X_SELECTED)
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	// configure DSP INT0=PIOX_IRQ-0 and INT1=PIOX_IRQ-1
	TP6X_set_mirq_sel(1,TP6X_MIRQ_SEL_XIRQ3);
#endif

	_REMOVE_PIOX_RESET;			// release PIOX reset
	
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	// configure C6x on-chip IRQs #4/#5 to ext INT-0/1
	C6X_INTMUX_SET(5,C6X_INTSEL_XINT5);
}



	// set PFG to 1MHz = (25MHz*2*(V=20)/((R=100)*(S=10)*(X=1))
#define PFG_V_DATA		20
#define PFG_R_DATA      	100
#define PFG_S_DATA		TPDAS_A65M_S_DIV10
#define PFG_X_DATA		TPDAS_A65M_X_DIV1

	// define ADC input channel
#define	ADC_IMUX		TPDAS_A65M_ADC_IMUX_CH1


#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency

	// DAC data array length
#define	_DAC_ARRAY_LEN		0x200

		// DAC-1/2 data array pointers
#define	_DAC1_DATA_PTR		(0x80008000)	
#define	_DAC2_DATA_PTR		(0x80008000+(_DAC_ARRAY_LEN*4))	

#define	_DAC_PAE_OFFSET		0x800
#define	_DAC_PAF_OFFSET		0x1000


main()
{ int i;
  int *mptr1, *mptr2;

	init_cpu();


	TPDAS_A65M_SET_HIRQ0_OFF;	// disable HIRQ-0 in order to retrigger on IRT

		// configure PFG, Fs selector and ADC DAQ SYNC
	TPDAS_A65M_SET_FS_SEL_PFG;
	TPDAS_A65M_SET_DAC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_SYNC_MODE_MASTER;

	TPDAS_A65M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	TPDAS_A65M_set_daq_mux_rg(ADC_IMUX);
	TPDAS_A65M_SET_DSOUT_SEL_DAC;
	TPDAS_A65M_SET_DAC_FMT_12;


        		// now reset DAC FIFO and program FIFO PAF/PAE flags
	if(   TPDAS_A65M_set_dac_fifo_offset(_DAC_PAE_OFFSET,_DAC_PAF_OFFSET)!=0
	  )
            goto error;

			// set DAC FIFO DAQ mode with PAE termination flag and DAC-IRT PAF flag
	TPDAS_A65M_SET_DAC_MODE_PTM;
	TPDAS_A65M_RESET_DAC_DAQ;     		// perform DAC FIFO/DAQ reset
	TPDAS_A65M_SET_DAC_DAQ_TF_SEL_PAE;
	TPDAS_A65M_SET_DAC_IRT_TF_SEL_PAF;
	TPDAS_A65M_CLR_ERR_ALL;			// clear all errors if they exist

	if(   TPDAS_A65M_dac1_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac1_fifo_pae_data() ==0
	   || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac1_fifo_paf_data() !=0
           || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac2_fifo_pae_data() ==0
	   || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac2_fifo_paf_data() !=0
	   || TPDAS_A65M_dac_irt_err_data()   !=0       
	   || TPDAS_A65M_dac_fifo_unf_data()  !=0
	   || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

			// now fill-in DAC-1/2 data arrays and transfer them to DAC-1/2 using DAC-IRT
	mptr1 = (int *) _DAC1_DATA_PTR;
	mptr2 = (int *) _DAC2_DATA_PTR;

	for(i=0;i<_DAC_ARRAY_LEN;i++)
	  { 		//-- fill-in DAC-1 as SINE signal
		*mptr1++ = (int)((float)  (2000.0 * sin(6.28 *((float)(i%_DAC_ARRAY_LEN))/((float)_DAC_ARRAY_LEN)))) ;
		
			//-- fill-in DAC-2 as RAMP signal
		*mptr2++ = (int)((float)  (2000.0 * ((float)((i%_DAC_ARRAY_LEN)-(_DAC_ARRAY_LEN/2)))/((float)(_DAC_ARRAY_LEN/2)))) ;
	  }



			// deselect source at PIOX IRQ-0 line
	TPDAS_A65M_SET_HIRQ0_EN_ON;
	TPDAS_A65M_SET_HIRQ0_SEL_OFF;

/*  configure DMA-0/1 to transfer DAC-1/2 data via EXT_INT4 DSP interrupt (PIOX IRQ-0)
    in auto-init mode in oder to exclude successive reloads of DMA configuration during
    continuous DAC PTM mode:
	- SRC DIR = +1, DST DIR = no
	- SRC SYNC = no, DST SYNC = EXT_INT4
	- esize = 32-bit words
	- PRI = DMA over CPU
	- SRC RELOAD = from global address B/C
	- DST RELOAD = NO 
	- CNT RELOAD = from global counter rg A
	- activate RSYNC/WSYNC events in case they have been missed

   PCRG @ DMA-0:   00 01 1 0 0 1 00100 00000 0 0 00 00 00 01 00 00	
   PCRG @ DMA-1:   00 10 1 0 0 1 00100 00000 0 0 00 00 00 01 00 00	
*/

	*C6X_DMA0_PCRG=0x19200010;	
	*C6X_DMA0_SCRG=0x30000;
	*C6X_DMA0_SARG=_DAC1_DATA_PTR;
	*C6X_DMA0_DARG=TPDAS_A65M_DAC1_FIFO_DATA_RG;
	*C6X_DMA0_TCRG=_DAC_ARRAY_LEN;

	*C6X_DMA1_PCRG=0x29200010;	
	*C6X_DMA1_SCRG=0;
	*C6X_DMA1_SARG=_DAC2_DATA_PTR;
	*C6X_DMA1_DARG=TPDAS_A65M_DAC2_FIFO_DATA_RG;
	*C6X_DMA1_TCRG=_DAC_ARRAY_LEN;

		// global DMA register for auto-init reload:
	*C6X_DMA0_GCRARG= _DAC_ARRAY_LEN;	// counter reload (DMA-0/1)
	*C6X_DMA0_GABRG = _DAC1_DATA_PTR;	// source address DMA-0
	*C6X_DMA0_GACRG = _DAC2_DATA_PTR;	// source address DMA-1

		// enable PIOX IRQ-0 line and select DAC-IRT irq sources
		// note!!!: 
		//	T6x/E6x require negative DAC IRT signal,
		//      whereas TP6x require positive DAC IRT    
#if(_TP6X_SELECTED)
	TPDAS_A65M_SET_HIRQ0_DAC_IRT_POS;
#else
	TPDAS_A65M_SET_HIRQ0_DAC_IRT_NEG;
#endif

	*C6X_DMA0_PCRG|=0x3; // start DMA0 in auto-init mode
	*C6X_DMA1_PCRG|=0x3; // start DMA1 in auto-init mode

		// wait until DAC1-FIFO_PAF and DAC2-FIFO_PAF are set in DAC-IRT. 
		// Check for errors and exit on any error.
	while(TPDAS_A65M_dac1_fifo_paf_data()==0 || TPDAS_A65M_dac2_fifo_paf_data()==0)
         { if(   TPDAS_A65M_dac_irt_err_data()   !=0
	      || TPDAS_A65M_dac_fifo_unf_data()  !=0
	      || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	     )
                 goto error;
	 }    
		// enable DAC-DAQ
	TPDAS_A65M_SET_DAC_DAQ_RUN;

	if(   TPDAS_A65M_dac_daq_run_data() == 0
	   || TPDAS_A65M_dac_daq_end_data() != 0
	  ) goto error;
	

		// loop here infinitely with DMA auto-init
		// check for errors during DAQ. Abort on any error.
	while(1)
         { if(   TPDAS_A65M_dac_irt_err_data()   !=0
	      || TPDAS_A65M_dac_fifo_unf_data()  !=0
	      || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	     )
                 goto error;
	 }

    error:          	   // trap here on any error
	 *C6X_DMA0_PCRG=0; // stop DMA0
         *C6X_DMA1_PCRG=0; // stop DMA1
         TPDAS_A65M_SET_DAC_DAQ_ABORT;
    	while(1);	


}


