/********************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Demo Utilities. Rev.1A
  OPM XDSIN/XDSOUT test for TORNADO-6x/P6x/E6x.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


   Description:
   ------------
	This demo performs continuous DAQ test for 32-bit XDSIN and 32-bit XDSOUT in the 
	OPM DAQ mode as the following:
	  - on-board PFG 65MHz is used as sampling frequecy source
	  - DSIN-MUX and DSOUT-MUX data stream multiplexers are configured to XDSIN and XDSOUT
	  - DAC-1/DAC-2 FIFO are filled in with the test pattern data and are loaded using 
	    host DSP software. DAC data arrays are swapped in each cycle.
	    DAC-1/2 FIFO data are transferred into C6x DSP on-chip data memory and 
	    can be veiwed via Code Composer memory viewer:
		- DAC-1 data are available at the xdsout_a_data[] array base address
		- DAC-2 data are available at the xdsout_b_data[] array base address
	  - ADC/DAC s/w synchronization is used
	  - DAC FIFO PAF flag are used as DAQ TF
	  - DAC/ADC FIFO PAF flag are used as DAQ TF
	  - ADC FIFO data are downloaded to DSP on-chip memory under host DSP software control.
	  - ADC-1/2 FIFO data are transferred into C6x DSP on-chip data memory and 
	    can be veiwed via Code Composer memory viewer:
		- ADC-1 data are available at 0x80008000 address
		- ADC-2 data are available at 0x80008100 address
	  - GPIO-0 is configured as the output and can be used as successful progress 
	    indicator. If test is running OK, then GPIO-0 is toggled in each cycle.
	

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. 

	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board. TI CCS tools are required to 
	recompile this demo.


  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/P6x mainboard
***********************************************************************/


#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#include	"math.h"
#include	"c6x_def.h"

//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#include			"te6x_dsp.h"
#define _REMOVE_PIOX_RESET	TE6X_set_pxsx_run(0x3)
#define	_PIOX16_SADDR_		TE6X_PIOX_SADDR
#endif

#if(_T6X_SELECTED)
#include			"t6x_dsp.h"
#define _REMOVE_PIOX_RESET	T6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		T6X_PIOX_SADDR
#endif

#if(_TP6X_SELECTED)
#include			"tp6x_dsp.h"
#define _REMOVE_PIOX_RESET	TP6X_set_pxsx_run(0x7)
#define	_PIOX16_SADDR_		TP6X_PIOX_SADDR
#endif
//--------------------------------------------------------------

// define host DSP platform for PA65_HD.H header file
#define _HOST_C6X_DSP_		1

//~~~~~~~~~~ include T/PDAS DCM header file ~~~~~~~~~~~~~~~~~~~~
#include	"pa65_hd.h"	
#include	"pa65x_hd.h"	
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



// perform host TORNADO specific EMIF initialization
init_cpu()
{
#if(_E6X_SELECTED)
	*C6X_EMIF_GCR=TE6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TE6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TE6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TE6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TE6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TE6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TE6X_EMIF_CE3_DATA;
#endif

#if(_T6X_SELECTED)
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif

#if(_TP6X_SELECTED)
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

	TP6X_set_mirq_sel(0,TP6X_MIRQ_SEL_XIRQ2);	// configure DSP INT0=PIOX_IRQ-0 and INT1=PIOX_IRQ-1
	TP6X_set_mirq_sel(1,TP6X_MIRQ_SEL_XIRQ3);
#endif

	_REMOVE_PIOX_RESET;			// release PIOX reset
	
	C6X_INTMUX_SET(4,C6X_INTSEL_XINT4);	// configure C6x on-chip IRQs #4/#5 to ext INT-0/1
	C6X_INTMUX_SET(5,C6X_INTSEL_XINT5);
}



	// set PFG to 65MHz = (25MHz*2*(V=130)/((R=50)*(S=2)*(X=1))
#define PFG_V_DATA		130
#define PFG_R_DATA      	50
#define PFG_S_DATA		TPDAS_A65M_S_DIV2
#define PFG_X_DATA		TPDAS_A65M_X_DIV1

	// define ADC input channel
#define	ADC_IMUX		TPDAS_A65M_ADC_IMUX_CH1


#define	_wait_pfg_lock()	for(i=0;i<1000;i++)	// delay to allow PFG to lock output frequency


unsigned xdsout_a_data[]= { 0x5555,0xaaaa,0x0000,0xffff,
			    0x0001,0x0002,0x0004,0x0008,
			    0x0010,0x0020,0x0040,0x0080,
			    0x0100,0x0200,0x0400,0x0800,
			    0x1000,0x2000,0x4000,0x8000,
			    0x55aa,0xaa55,0x5a5a,0xa5a5,0x5aa5,0xa55a,
			    0x1234,0x4321,0x9735,0xe913,0x74f1,0x2410,
			    0,0,0,0,0,0,0,0 };
			
unsigned xdsout_b_data[]= { 0xaaaa,0x5555,0xaaaa,0xffff,
			    0x8000,0x4000,0x2000,0x1000,
			    0x0800,0x0400,0x0200,0x0100,
			    0x0080,0x0040,0x0020,0x0010,
			    0x0008,0x0004,0x0002,0x0001,
			    0xfedc,0xba98,0x3159,0xf017,0x534b,0x3ae1,
			    0x55aa,0xaa55,0x5a5a,0xa5a5,0x5aa5,0xa55a,
			    0,0,0,0,0,0,0,0 };


		// DAC data length
#define	_DAC_DAQ_LEN		38

		// extra ADC data array length due to the missing initial DAC FIFO samples
		// and resync of DAC output data
#define _ADC_EXTRA_DATA_LEN_MIN	4
#define _ADC_EXTRA_DATA_LEN_MAX	6

		// ADC data length
#define _ADC_DAQ_LEN		(_DAC_DAQ_LEN-8)

		// ADC data pointers
#define	_ADC1_DATA_PTR		(0x80008000)
#define	_ADC2_DATA_PTR		(0x80008000+0x100)


main()
{ int i, pass_counter, adc_header_detected;
  unsigned *dac1_data_ptr, *dac2_data_ptr, *mptr1, *mptr2, *adc1_ptr, *adc2_ptr;

	init_cpu();

		// configure PFG, Fs selector and ADC DAQ SYNC
	TPDAS_A65M_SET_FS_SEL_PFG;
	TPDAS_A65M_SET_ADC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_DAC_SYNC_SEL_SFT;
	TPDAS_A65M_SET_SYNC_MODE_MASTER;

	TPDAS_A65M_SET_PFG_EXACTLY(PFG_V_DATA,PFG_R_DATA,PFG_S_DATA,PFG_X_DATA);
	_wait_pfg_lock();

	TPDAS_A65M_SET_DSIN_SEL_XDSIN;
	TPDAS_A65M_SET_DSOUT_SEL_XDSOUT;
	TPDAS_A65M_SET_ADC_FMT_12;
	TPDAS_A65M_SET_DAC_FMT_12;

	TPDAS_A65M_RESET_DAQ;		// reset DAQ/FIFO and clear all errors if they exist

        		// now reset ADC/DAC FIFO and program FIFO PAF/PAE flags
	if(   TPDAS_A65M_set_adc_fifo_offset(_ADC_DAQ_LEN/2,_ADC_DAQ_LEN+_ADC_EXTRA_DATA_LEN_MAX)!=0
	   || TPDAS_A65M_set_dac_fifo_offset(_ADC_DAQ_LEN/2,_DAC_DAQ_LEN)!=0
	  )
            goto error;

	if(   TPDAS_A65M_adc1_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc1_fifo_pae_data() ==0
	   || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc1_fifo_paf_data() !=0
           || TPDAS_A65M_adc2_fifo_ef_data()  ==0
	   || TPDAS_A65M_adc2_fifo_pae_data() ==0
	   || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	   || TPDAS_A65M_adc2_fifo_paf_data() !=0
	  )
            goto error;

	if(   TPDAS_A65M_dac1_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac1_fifo_pae_data() ==0
	   || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac1_fifo_paf_data() !=0
           || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	   || TPDAS_A65M_dac2_fifo_pae_data() ==0
	   || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	   || TPDAS_A65M_dac2_fifo_paf_data() !=0
	  )
            goto error;

	if(   TPDAS_A65M_adc_irt_err_data()   !=0
	   || TPDAS_A65M_dac_irt_err_data()   !=0       
	   || TPDAS_A65M_adc_fifo_ovf_data()  !=0
	   || TPDAS_A65M_dac_fifo_unf_data()  !=0
	   || TPDAS_A65M_adc_fifo_unf_data()  !=0
	   || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

			// set ADC FIFO DAQ mode with PAF termination flag
			// set DAC FIFO DAQ mode with EF termination flag
			// and initialize DAQ procedure
	TPDAS_A65M_SET_ADC_MODE_OPM;
	TPDAS_A65M_SET_DAC_MODE_OPM;

	TPDAS_A65M_SET_ADC_DAQ_TF_SEL_PAF;
	TPDAS_A65M_SET_DAC_DAQ_TF_SEL_EF;
	TPDAS_A65M_SET_ADC_IRT_TF_SEL_EF;
	TPDAS_A65M_SET_DAC_IRT_TF_SEL_PAF;

		// configure GPIO-0 as output:
		//  GPIO-0 will indicate the pass counter value
	TPDAS_A65M_SET_GPIO0_DIR_OUTPUT;
	TPDAS_A65M_SET_GPIO0_DATA_OFF;

	pass_counter=0;

		// infinite XDSIN/XDSOUT test loop
    while(1)
     {
		// output progress indicator via GPIO-0 pin
	TPDAS_A65M_SET_GPIO0_DATA(pass_counter);

			// perform ADC/DAC FIFO/DAQ reset
	TPDAS_A65M_RESET_DAQ;     

			// now upload DAC-1/2 FIFOs
	dac1_data_ptr = (int *) (pass_counter==0)? xdsout_a_data:xdsout_b_data ;
	dac2_data_ptr = (int *) (pass_counter==0)? xdsout_b_data:xdsout_a_data ;

	mptr1 = dac1_data_ptr;
	mptr2 = dac2_data_ptr;

	for(i=0;i<_DAC_DAQ_LEN;i++)
	{
	  TPDAS_A65M_set_xdsout1(*mptr1++);
	  TPDAS_A65M_set_xdsout2(*mptr2++);

			// verify DAC FIFO flag procurement
	  if(    ((i+1)<_DAC_DAQ_LEN  && TPDAS_A65M_dac1_fifo_paf_data()!=0)
             ||  ((i+1)>=_DAC_DAQ_LEN && TPDAS_A65M_dac1_fifo_paf_data()==0)
             ||  TPDAS_A65M_dac1_fifo_ff_data()!=0
	    ) 
		goto error;

	  if(    ((i+1)<_DAC_DAQ_LEN  && TPDAS_A65M_dac2_fifo_paf_data()!=0)
             ||  ((i+1)>=_DAC_DAQ_LEN && TPDAS_A65M_dac2_fifo_paf_data()==0)
             ||  TPDAS_A65M_dac2_fifo_ff_data()!=0
	    ) 
		goto error;
	}


	if(    TPDAS_A65M_dac1_fifo_ef_data()  !=0
	    || TPDAS_A65M_dac1_fifo_pae_data() !=0
	    || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac1_fifo_paf_data() ==0
            || TPDAS_A65M_dac2_fifo_ef_data()  !=0
	    || TPDAS_A65M_dac2_fifo_pae_data() !=0
	    || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac2_fifo_paf_data() ==0
	  )
            goto error;

	if(    TPDAS_A65M_adc1_fifo_ef_data()  ==0
	    || TPDAS_A65M_adc1_fifo_pae_data() ==0
	    || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc1_fifo_paf_data() !=0
            || TPDAS_A65M_adc2_fifo_ef_data()  ==0
	    || TPDAS_A65M_adc2_fifo_pae_data() ==0
	    || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc2_fifo_paf_data() !=0
	  )
            goto error;


			// now initialize ADC/DAC DAQ processes
	TPDAS_A65M_set_daq_cntr2_rg(TPDAS_A65M_ADC_DAQ_RUN | TPDAS_A65M_DAC_DAQ_RUN);

			// wait until ADC/DAC DAQ terminates
	while(    TPDAS_A65M_adc_daq_end_data()==0
	       || TPDAS_A65M_dac_daq_end_data()==0
	     );


	if(    TPDAS_A65M_adc_fifo_ovf_data()  !=0
	    || TPDAS_A65M_adc_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_unf_data()  !=0
	    || TPDAS_A65M_dac_fifo_ovf_data()  !=0
	  )
            goto error;

	if(    TPDAS_A65M_dac1_fifo_ef_data()  ==0
	    || TPDAS_A65M_dac1_fifo_pae_data() ==0
	    || TPDAS_A65M_dac1_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac1_fifo_paf_data() !=0
            || TPDAS_A65M_dac2_fifo_ef_data()  ==0
	    || TPDAS_A65M_dac2_fifo_pae_data() ==0
	    || TPDAS_A65M_dac2_fifo_ff_data()  !=0
	    || TPDAS_A65M_dac2_fifo_paf_data() !=0
	  )
            goto error;

	if(    TPDAS_A65M_adc1_fifo_ef_data()  !=0
	    || TPDAS_A65M_adc1_fifo_pae_data() !=0
	    || TPDAS_A65M_adc1_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc1_fifo_paf_data() ==0
            || TPDAS_A65M_adc2_fifo_ef_data()  !=0
	    || TPDAS_A65M_adc2_fifo_pae_data() !=0
	    || TPDAS_A65M_adc2_fifo_ff_data()  !=0
	    || TPDAS_A65M_adc2_fifo_paf_data() ==0
	  )
            goto error;


			// read and store ADC FIFO data
			// Note: input FIFO data until the match with the 1st DAC word must
			//       be ignored due to possible 1-ccl mis-synchronizations
			//       between ADC and DAC DAQ controllers
			
	adc1_ptr = (unsigned *) _ADC1_DATA_PTR;
	adc2_ptr = (unsigned *) _ADC2_DATA_PTR;
	mptr1 = dac1_data_ptr;
	mptr2 = dac2_data_ptr;
	adc_header_detected=0;

	for(i=0;i<(_ADC_DAQ_LEN+_ADC_EXTRA_DATA_LEN_MAX);i++)
		{ 
				// check for ADC FIFO is still not empty
			if(    TPDAS_A65M_adc1_fifo_ef_data()!=0
            	    	    || TPDAS_A65M_adc2_fifo_ef_data()!=0
	  	  	  )
	            	    goto error;

				// read and store ADC FIFO data
			*adc1_ptr = TPDAS_A65M_xdsin1_data();
			*adc2_ptr = TPDAS_A65M_xdsin2_data();

				// check for header not start too early
		        if(     i<(_ADC_EXTRA_DATA_LEN_MIN-1)
			     && adc_header_detected!=0
			  )
				goto error;
			
				// check for ADC header has not been detected,
				// and detect it if not yet
			if(adc_header_detected==0)
			  {
				// validate that header limits are not exceeded
			    if( i>=_ADC_EXTRA_DATA_LEN_MAX)
				 goto error;

				// compare ADC1 to DAC1 and ADC2 to DAC2 
			    if(    (*adc1_ptr)==(*mptr1)
				&& (*adc2_ptr)==(*mptr2)
			      )
			        adc_header_detected=1;
			  }

				// increment ADC save pointers and counter 
				// if header has been detected
			if(adc_header_detected!=0)				
			  {		
				adc1_ptr++;
				adc2_ptr++;
			  }
		}		


	if(    TPDAS_A65M_adc1_fifo_ef_data()  ==0
       	    || TPDAS_A65M_adc2_fifo_ef_data()  ==0
 	  )
            goto error;


			// verify ADC FIFO data against DAC FIFO data
	adc1_ptr = (unsigned *) _ADC1_DATA_PTR;
	adc2_ptr = (unsigned *) _ADC2_DATA_PTR;
	mptr1 = dac1_data_ptr;
	mptr2 = dac2_data_ptr;

	for(i=0; i<_ADC_DAQ_LEN; i++,mptr1++,mptr2++,adc1_ptr++,adc2_ptr++)
	  {
		if(    (*adc1_ptr) != (*mptr1)
		   ||  (*adc2_ptr) != (*mptr2)
		  )
		    goto error;
	  }



	pass_counter++;
	pass_counter %=2 ;
     }		


    error:              	// trap here on any error
        TPDAS_A65M_RESET_DAQ;	// reset DAQ/IRT controllers
    	while(1);	


}


