/************************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Utilities. Rev.1A
  Optional host DSP functions for TORNADO DSP systems/controllers with C3x/C6x DSP.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


  Notes:
  ------
    1. This file contains optional functions for ADC/DAC FIFO programming, etc.
    2. This file must be used in conjunction with main PA65_HD.H file 
       for C3x/C6x DSP environment of host TORNADO-3x/P3x/E3x/6x/P6x/E6x DSP systems
       and controllers. This file must be included into only .C module of host
       DSP software in order to exclude function redefinition.
*************************************************************************/




//******************** FIFO programming ****************************

	//--- set FIFO PAF/PAE offset values
	// ADC/DAC FIFO PAE/PAF params values are assumed to be the values
	// origined to the FIFO address #0.
	//
	// Note different PAE/PAF flags adjustment for ADC and DAC FIFOs in order
	// to qrite/read exactly specified number of FIFO samples.

int TPDAS_A65M_set_adc_fifo_offset(unsigned pae, unsigned paf)
{ int i;
  unsigned d;
        	//-- validate input params
	if((paf+2)>=TPDAS_A65M_FIFO_LEN  || pae>=TPDAS_A65M_FIFO_LEN) return(-1);

		// set ADC FIFO CNF mode and perform FIFO reset
        TPDAS_A65M_SET_ADC_MODE_CNF;	
	TPDAS_A65M_RESET_ADC_DAQ;

		// load 18-bit PAE data starting fm LSB
		// Note!!!
		//   Offset value for the PAE flag, which has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO address #0, so input PAE function parameter
		//   is OK to be transmitted directly to FIFO.
		//   Output FIFO PAE flag will be set active during
		//   writing of PAE samples to FIFO and will be set inactive
		//   after writing the (PAE+1)-th FIFO sample (in case
		//   not FIFO reads is perfromed). Output FIFO PAE flag
		//   is updated on FIFO RCLK. 

       	d = pae;
	for(i=0;i<18; i++)
         {
           TPDAS_A65M_set_adc_fifo_sin(d) ;
	   d >>= 1;
	 }


		// load 18-bit PAF data starting fm LSB
		// Note!!!
		//   FIFO chip requires that PAF offset value has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO LEN, and not to the FIFO address #0. So,
		//   input PAF function parameter has to be converted
		//   to (FIFO_LEN-PAF) prior transmision to FIFO.
		//   Due to continuous WCLK, the output FIFO PAF flag
		//   will be set active after 2 WCLK cycles after writing
		//   (FIFO_LEN-PAF) FIFO samples, so the programmed value
		//   must be (PAF-2).

	d = TPDAS_A65M_FIFO_LEN-paf+2;
	for(i=0;i<18; i++)
         {
           TPDAS_A65M_set_adc_fifo_sin(d) ;
	   d >>= 1;
	 }

		// exit OK
	return(0);
}



int TPDAS_A65M_set_dac_fifo_offset(unsigned pae, unsigned paf)
{ int i;
  unsigned d;
        	//-- validate input params
	if(paf>=TPDAS_A65M_FIFO_LEN  || (pae+2)>=TPDAS_A65M_FIFO_LEN) return(-1);

		// set DAC FIFO CNF mode and perform FIFO reset
        TPDAS_A65M_SET_DAC_MODE_CNF;	
	TPDAS_A65M_RESET_DAC_DAQ;

		// load 18-bit PAE data starting fm LSB
		// Note!!!
		//   FIFO chip requires that PAE offset value must be origined to the
		//   FIFO address #0, so input PAE function parameter
		//   is OK to be transmitted directly to FIFO.
		//   Output FIFO PAE flag will be set active during
		//   writing of PAE samples to FIFO and will be set inactive
		//   after writing the (PAE+1)-th FIFO sample (in case
		//   not FIFO reads is perfromed). Due to continuous REN
		//   control, the output DAC FIFO PAE flag is updated after
		//   2 clocks of FIFO RCLK after active condition, so the 
		//   programmed value must be (PAE-2)

       	d = pae-2;
	for(i=0;i<18; i++)
         {
           TPDAS_A65M_set_dac_fifo_sin(d) ;
	   d >>= 1;
	 }


		// load 18-bit PAF data starting fm LSB
		// Note!!!
		//   FIFO chip requires that PAF offset value has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO LEN, and not to the FIFO address #0. So,
		//   input PAF function parameter has to be converted
		//   to (FIFO_LEN-PAF) prior transmision to FIFO.
		//   The output FIFO PAF flag will be set active after writing
		//   (FIFO_LEN-PAF) FIFO samples and is updated on DAC FIFO WCLK.

	d = TPDAS_A65M_FIFO_LEN-paf;
	for(i=0;i<18; i++)
         {
           TPDAS_A65M_set_dac_fifo_sin(d) ;
	   d >>= 1;
	 }

		// exit OK
	return(0);
}


//******************************************************************

