/************************************************************************
  T/PDAS-AD8/12D/65M-DA2/12D/65M PIOX-16 AD/DA DCM Utilities. Rev.1A
  Host DSP defs for TORNADO DSP systems/controllers with C3x/C6x DSP.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


  Notes:
  ------
    1. This file must be used as .H header file for C3x/C6x DSP
       environment of host TORNADO-3x/P3x/E3x/6x/P6x/E6x DSP systems
       and controllers.
    2. All commands are adjusted for 32-bit data words and
       occupy least significant 16-bit word.
*************************************************************************/


//!!!!! host PIOX-16 base address
// note:
//   1. _PIOX16_SADDR_ constant must be set to PIOX-16 base address
//   	in parent .C file in accordance with the DSP environment of the
//   	corresponding TORNADO DSP system/controller (refer to the .H
//   	file and documentation for your TORNADO DSP system/controller).
//   2. In case _PIOX16_SADDR_ constant is not defined in parent
//	.C file, then it is set to default PIOX-16 base address for  
//	TORNADO-3x/P3x/E3x DSP systems and controllers.

#ifndef _PIOX16_SADDR_
#define	_PIOX16_SADDR_		0xff0000	// default for T3x/TP3x/E3x 
#endif

//!!!!! host TORNADO DSP platform definition
// note:
//   1. _HOST_C6X_DSP_ constant must be defined to any value in parent .C file for 
//	any of host T6x/TP6x/E6x DSP platforms.
//   2. _HOST_C3X_DSP_ constant must be defined to any value in parent .C file for 
//	any of host T3x/TP3x/E3x DSP platforms.
//   3. In case none of _HOST_C6X_DSP_ and _HOST_C3X_DSP_ is defined
//	in parent .C file, then it C3x DSP is assumed to be host DSP.

	// check for host C6x DSP platform
#ifdef _HOST_C6X_DSP_		
		// treat C3X def as erroneous def
#ifdef _HOST_C3X_DSP_		
#undef _HOST_C3X_DSP_		
#endif
		// word address step definition
#define	_WAS_				4		
#endif

	// check for host C3x DSP platform
#ifdef _HOST_C3X_DSP_		
		// treat C6X def as erroneous def
#ifdef _HOST_C6X_DSP_		
#undef _HOST_C6X_DSP_		
#endif
		// word address step definition
#define	_WAS_				1		
#endif

	// check for both C3X/C6X defs are undefined, and assume C3X is host DSP
#ifndef _HOST_C6X_DSP_		
#ifndef _HOST_C3X_DSP_		
#define _HOST_C3X_DSP_		1		
		// word address step definition
#define	_WAS_				1		
#endif
#endif

//=========================== general defs ==============================

#define	TPDAS_A65M_OFF			0
#define	TPDAS_A65M_ON			1
	
//===============================================================================


//============== host PIOX I/O ports address definitions ========================

#define TPDAS_A65M_DAQ_CNTR1_RG		(_PIOX16_SADDR_+(0x0000*_WAS_))	// r/w: CONTROL #1 register
#define TPDAS_A65M_DAQ_CNTR2_RG		(_PIOX16_SADDR_+(0x0001*_WAS_))	// r/w: CONTROL #2 register
#define TPDAS_A65M_DAQ_CNTR3_RG		(_PIOX16_SADDR_+(0x0002*_WAS_))	// r/w: CONTROL #3 register
#define TPDAS_A65M_DAQ_SYNC_RG		(_PIOX16_SADDR_+(0x0003*_WAS_))	// r/w: CLOCK SELECTOR register
#define TPDAS_A65M_DAQ_MUX_RG		(_PIOX16_SADDR_+(0x0004*_WAS_))	// r/w: Data Streams and ADC MUX register

#define TPDAS_A65M_ADC_FIFO_STAT_RG	(_PIOX16_SADDR_+(0x0006*_WAS_))	// r:   FIFO/ADC status register
#define TPDAS_A65M_DAC_FIFO_STAT_RG	(_PIOX16_SADDR_+(0x0007*_WAS_))	// r:   FIFO/DAC status register

#define TPDAS_A65M_ERR_STAT_RG		(_PIOX16_SADDR_+(0x0008*_WAS_))	// r:   error status register
#define TPDAS_A65M_ERR_CLR_RG		(_PIOX16_SADDR_+(0x0008*_WAS_))	// w:   error clear register

#define TPDAS_A65M_PFG_CNTR1_RG		(_PIOX16_SADDR_+(0x000A*_WAS_))	// r/w: PFG CONTROL #1 register
#define TPDAS_A65M_PFG_CNTR2_RG		(_PIOX16_SADDR_+(0x000B*_WAS_))	// r/w: PFG CONTROL #2 register
#define TPDAS_A65M_PFG_CNTR3_RG		(_PIOX16_SADDR_+(0x000C*_WAS_))	// r/w: PFG CONTROL #3 register

#define TPDAS_A65M_GPIO_DATA_RG		(_PIOX16_SADDR_+(0x000E*_WAS_))	// r/w: GPIO data register
#define TPDAS_A65M_GPIO_DIR_RG		(_PIOX16_SADDR_+(0x000F*_WAS_))	// r/w: GPIO direction register

#define TPDAS_A65M_HIRQ0_SEL_RG		(_PIOX16_SADDR_+(0x0010*_WAS_))	// r/w: PIOX-16 HIRQ-0 selector register
#define TPDAS_A65M_HIRQ1_SEL_RG		(_PIOX16_SADDR_+(0x0011*_WAS_))	// r/w: PIOX-16 HIRQ-1 selector register
#define TPDAS_A65M_HIRQ2_SEL_RG		(_PIOX16_SADDR_+(0x0012*_WAS_))	// r/w: PIOX-16 HIRQ-2 selector register
#define TPDAS_A65M_HIRQ3_SEL_RG		(_PIOX16_SADDR_+(0x0013*_WAS_))	// r/w: PIOX-16 HIRQ-3 selector register

#define TPDAS_A65M_XIM_ERR_RG		(_PIOX16_SADDR_+(0x0016*_WAS_))	// r/w: PIOX-16 ERROR Aux IM register for HIRQ

#define TPDAS_A65M_ADC1_FIFO_DATA_RG	(_PIOX16_SADDR_+(0x8000*_WAS_))	// r:   FIFO/ADC1 data register
#define TPDAS_A65M_ADC2_FIFO_DATA_RG	(_PIOX16_SADDR_+(0x8001*_WAS_))	// r:   FIFO/ADC2 data register
#define TPDAS_A65M_DAC1_FIFO_DATA_RG	(_PIOX16_SADDR_+(0x8004*_WAS_))	// w:   FIFO/DAC1 data register
#define TPDAS_A65M_DAC2_FIFO_DATA_RG	(_PIOX16_SADDR_+(0x8005*_WAS_))	// w:   FIFO/DAC2 data register

#define TPDAS_A65M_ADC_FIFO_SIN_RG	(_PIOX16_SADDR_+(0x8002*_WAS_))	// w:   FIFO/ADC Serial-IN register for CNF mode
#define TPDAS_A65M_DAC_FIFO_SIN_RG	(_PIOX16_SADDR_+(0x8006*_WAS_))	// w:   FIFO/DAC Serial-IN register for CNF mode

#define TPDAS_A65M_ADC_DAQ_RESET_RG	(_PIOX16_SADDR_+(0x8008*_WAS_))	// w:   ADC DAQ controller reset register
#define TPDAS_A65M_DAC_DAQ_RESET_RG	(_PIOX16_SADDR_+(0x8009*_WAS_))	// w:   DAC DAQ controller reset register
#define TPDAS_A65M_DAQ_RESET_RG		(_PIOX16_SADDR_+(0x800A*_WAS_))	// w:   ADC+DAC DAQ controller reset register
//================================================================================


//========================== system parameters ===================================

#define	TPDAS_A65M_FIFO_LEN			0x40000L// ADC/DAC FIFO depth

#define TPDAS_A65M_PFG_REF_CLK_FREQ		25.0e+6	// 25MHz PFG source clk

//================================================================================

//============== control register bit/fields definitions =========================

#define TPDAS_A65M_CNTR_RG_DATA_BITMASK		0xff	// D0..D7 only are valid for all CONTROL registers	

	//-- Control #1 Register bits
#define TPDAS_A65M_ADC_MODE_BITMASK		0x3	// field bitmasks
#define TPDAS_A65M_DAC_MODE_BITMASK		0xc
#define TPDAS_A65M_ADC_FMT_BITMASK		0x30
#define TPDAS_A65M_DAC_FMT_BITMASK		0xc0

#define TPDAS_A65M_ADC_MODE_CNF			0x0	// FIFO-ADC MODE defs
#define TPDAS_A65M_ADC_MODE_PTM			0x2
#define TPDAS_A65M_ADC_MODE_OPM			0x3
#define TPDAS_A65M_DAC_MODE_CNF			0x0	// FIFO-DAC MODE defs		
#define TPDAS_A65M_DAC_MODE_PTM			0x8
#define TPDAS_A65M_DAC_MODE_OPM			0xc

#define TPDAS_A65M_ADC_FMT_1			0x0	// FIFO-ADC data format defs
#define TPDAS_A65M_ADC_FMT_2			0x10	
#define TPDAS_A65M_ADC_FMT_12			0x20	
#define TPDAS_A65M_DAC_FMT_1			0x0	// FIFO-DAC data format defs
#define TPDAS_A65M_DAC_FMT_2			0x40	
#define TPDAS_A65M_DAC_FMT_12			0x80	

#define TPDAS_A65M_ADDA_MODE_BITMASK		0x3	// aligned defs
#define TPDAS_A65M_ADDA_FMT_BITMASK		0x3
#define TPDAS_A65M_ADC_MODE_SHIFT		0x0	// field bit-shifts
#define TPDAS_A65M_DAC_MODE_SHIFT		0x2
#define TPDAS_A65M_ADC_FMT_SHIFT		0x4
#define TPDAS_A65M_DAC_FMT_SHIFT		0x6
#define TPDAS_A65M_ADDA_MODE_CNF		0x0	// universal FIFO MODE defs
#define TPDAS_A65M_ADDA_MODE_PTM		0x2
#define TPDAS_A65M_ADDA_MODE_OPM		0x3
#define TPDAS_A65M_ADDA_FMT_1			0x0	// universal FIFO data format defs
#define TPDAS_A65M_ADDA_FMT_2			0x1	
#define TPDAS_A65M_ADDA_FMT_12			0x2	

	//-- Control #2 Register bits
#define TPDAS_A65M_ADC_DAQ_RUN_BITMASK		0x1	// bits bitmasks
#define TPDAS_A65M_DAC_DAQ_RUN_BITMASK		0x2
#define TPDAS_A65M_ADC_DAQ_SYNC_BITMASK		0x10
#define TPDAS_A65M_DAC_DAQ_SYNC_BITMASK		0x20
#define TPDAS_A65M_ADC_DAQ_END_BITMASK		0x40
#define TPDAS_A65M_DAC_DAQ_END_BITMASK		0x80

#define TPDAS_A65M_ADC_DAQ_RUN			0x1	// DAQ control bits
#define TPDAS_A65M_DAC_DAQ_RUN			0x2	

#define TPDAS_A65M_ADC_DAQ_SYNC_OK		0x10	// r-only sync bits values
#define TPDAS_A65M_DAC_DAQ_SYNC_OK		0x20

#define TPDAS_A65M_ADC_DAQ_ABORT		0x40	// write-only ABORT command bits ('0' alue is ignored)	
#define TPDAS_A65M_DAC_DAQ_ABORT		0x80


	//-- Control #3 Register bits
#define TPDAS_A65M_ADC_DAQ_TF_SEL_BITMASK	0x1	// bits bitmasks
#define TPDAS_A65M_DAC_DAQ_TF_SEL_BITMASK	0x2	
#define TPDAS_A65M_ADC_IRT_TF_SEL_BITMASK	0x4	
#define TPDAS_A65M_DAC_IRT_TF_SEL_BITMASK	0x8	

#define TPDAS_A65M_ADC_DAQ_TF_SEL_FF		0x0	// ADC-DAQ termination flags
#define TPDAS_A65M_ADC_DAQ_TF_SEL_PAF		0x1	

#define TPDAS_A65M_DAC_DAQ_TF_SEL_EF		0x0	// DAC-DAQ termination flags
#define TPDAS_A65M_DAC_DAQ_TF_SEL_PAE		0x2	

#define TPDAS_A65M_ADC_IRT_TF_SEL_EF		0x0	// ADC IRT termination flags
#define TPDAS_A65M_ADC_IRT_TF_SEL_PAE		0x4	

#define TPDAS_A65M_DAC_IRT_TF_SEL_FF		0x0	// DAC IRT termination flags
#define TPDAS_A65M_DAC_IRT_TF_SEL_PAF		0x8	


	//-- DAQ SYNC Register bits
#define TPDAS_A65M_FS_SEL_BITMASK		0x3	// fields bitmasks
#define TPDAS_A65M_ADC_SYNC_SEL_BITMASK		0x10	
#define TPDAS_A65M_DAC_SYNC_SEL_BITMASK		0x20	
#define TPDAS_A65M_XSYNC_MODE_BITMASK		0x40	
#define TPDAS_A65M_SYNC_MODE_BITMASK		0x80	

#define TPDAS_A65M_FS_SEL_PFG			0x0	// Sampling frequency (Fs) selector
#define TPDAS_A65M_FS_SEL_XFS			0x1
#define TPDAS_A65M_FS_SEL_PXTM0			0x2	
#define TPDAS_A65M_FS_SEL_PXTM1			0x3

#define TPDAS_A65M_ADC_SYNC_SEL_SFT		0x00	// ADC DAQ synch selector
#define TPDAS_A65M_ADC_SYNC_SEL_XSYNC		0x10	

#define TPDAS_A65M_DAC_SYNC_SEL_SFT		0x00	// DAC DAQ synch selector
#define TPDAS_A65M_DAC_SYNC_SEL_XSYNC		0x20	

#define TPDAS_A65M_XSYNC_MODE_LEVEL		0x00	// ADC/DAC XSYNC edge/level mode selector
#define TPDAS_A65M_XSYNC_MODE_EDGE		0x40	

#define TPDAS_A65M_SYNC_MODE_MASTER		0x00	// ADC/DAQ sync mode
#define TPDAS_A65M_SYNC_MODE_SLAVE		0x80

#define TPDAS_A65M_ADC_SYNC_SEL_SHIFT		4	// aligned defs
#define TPDAS_A65M_DAC_SYNC_SEL_SHIFT		5	
#define TPDAS_A65M_ADDA_SYNC_SEL_SFT		0
#define TPDAS_A65M_ADDA_SYNC_SEL_XSYNC		1	


	//-- DAQ data MUX Register bits
#define TPDAS_A65M_ADC_IMUX_BITMASK		0x3	// fields bitmasks
#define TPDAS_A65M_DSIN_SEL_BITMASK		0x10	
#define TPDAS_A65M_DSOUT_SEL_BITMASK		0x20	

#define TPDAS_A65M_ADC_IMUX_CH0			0x0	// ADC input MUX
#define TPDAS_A65M_ADC_IMUX_CH1			0x1
#define TPDAS_A65M_ADC_IMUX_CH2			0x2
#define TPDAS_A65M_ADC_IMUX_CH3			0x3

#define TPDAS_A65M_DSIN_SEL_ADC			0x00	// Input data stream selector
#define TPDAS_A65M_DSIN_SEL_XDSIN		0x10

#define TPDAS_A65M_DSOUT_SEL_DAC		0x00	// Output data stream selector
#define TPDAS_A65M_DSOUT_SEL_XDSOUT		0x20


	//-- ADC/DAC FIFO status Registers bits
#define TPDAS_A65M_ADC1_FIFO_FF_BITMASK		0x01	// ADC-1 FIFO status flags
#define TPDAS_A65M_ADC1_FIFO_PAF_BITMASK	0x02
#define TPDAS_A65M_ADC1_FIFO_EF_BITMASK		0x04
#define TPDAS_A65M_ADC1_FIFO_PAE_BITMASK	0x08
#define TPDAS_A65M_ADC2_FIFO_FF_BITMASK		0x10	// ADC-2 FIFO status flags
#define TPDAS_A65M_ADC2_FIFO_PAF_BITMASK	0x20
#define TPDAS_A65M_ADC2_FIFO_EF_BITMASK		0x40
#define TPDAS_A65M_ADC2_FIFO_PAE_BITMASK	0x80

#define TPDAS_A65M_DAC1_FIFO_FF_BITMASK		0x01	// DAC-1 FIFO status flags
#define TPDAS_A65M_DAC1_FIFO_PAF_BITMASK	0x02
#define TPDAS_A65M_DAC1_FIFO_EF_BITMASK		0x04
#define TPDAS_A65M_DAC1_FIFO_PAE_BITMASK	0x08
#define TPDAS_A65M_DAC2_FIFO_FF_BITMASK		0x10	// DAC-2 FIFO status flags
#define TPDAS_A65M_DAC2_FIFO_PAF_BITMASK	0x20
#define TPDAS_A65M_DAC2_FIFO_EF_BITMASK		0x40
#define TPDAS_A65M_DAC2_FIFO_PAE_BITMASK	0x80

#define TPDAS_A65M_ADDA1_FIFO_FF_SHIFT		0	// aligned defs
#define TPDAS_A65M_ADDA1_FIFO_PAF_SHIFT		1
#define TPDAS_A65M_ADDA1_FIFO_EF_SHIFT		2
#define TPDAS_A65M_ADDA1_FIFO_PAE_SHIFT		3
#define TPDAS_A65M_ADDA2_FIFO_FF_SHIFT		4
#define TPDAS_A65M_ADDA2_FIFO_PAF_SHIFT		5
#define TPDAS_A65M_ADDA2_FIFO_EF_SHIFT		6
#define TPDAS_A65M_ADDA2_FIFO_PAE_SHIFT		7



	//-- ERROR status Register bits (r-only)
#define TPDAS_A65M_ADC_FIFO_OVF_BITMASK		0x01	// error bits
#define TPDAS_A65M_ADC_FIFO_UNF_BITMASK		0x02
#define TPDAS_A65M_DAC_FIFO_OVF_BITMASK		0x04
#define TPDAS_A65M_DAC_FIFO_UNF_BITMASK		0x08
#define TPDAS_A65M_ADC_IRT_ERR_BITMASK		0x10
#define TPDAS_A65M_DAC_IRT_ERR_BITMASK		0x20

#define TPDAS_A65M_ADC_FIFO_OVF_SHIFT		0	// aligned defs
#define TPDAS_A65M_ADC_FIFO_UNF_SHIFT		1
#define TPDAS_A65M_DAC_FIFO_OVF_SHIFT		2
#define TPDAS_A65M_DAC_FIFO_UNF_SHIFT		3
#define TPDAS_A65M_ADC_IRT_ERR_SHIFT		4
#define TPDAS_A65M_DAC_IRT_ERR_SHIFT		5


	//-- ERROR CLEAR Register bits (w-only, '0' bit values are ignored)
#define TPDAS_A65M_ADC_FIFO_OVF_CLR		0x01
#define TPDAS_A65M_ADC_FIFO_UNF_CLR		0x02
#define TPDAS_A65M_DAC_FIFO_OVF_CLR		0x04
#define TPDAS_A65M_DAC_FIFO_UNF_CLR		0x08
#define TPDAS_A65M_ADC_IRT_ERR_CLR		0x10
#define TPDAS_A65M_DAC_IRT_ERR_CLR		0x20


	//-- HIRQ-0..3 Interrupt control Registers bits
#define TPDAS_A65M_HIRQ_SEL_BITMASK		0x1f	// bitmasks
#define TPDAS_A65M_HIRQ_EN_BITMASK		0x80

#define TPDAS_A65M_HIRQ_SEL_NO			0x00	// HIRQ selector
#define TPDAS_A65M_HIRQ_SEL_ADC_DAQ_END		0x01
#define TPDAS_A65M_HIRQ_SEL_DAC_DAQ_END		0x02
#define TPDAS_A65M_HIRQ_SEL_ADC_DAQ_SYNC	0x03
#define TPDAS_A65M_HIRQ_SEL_DAC_DAQ_SYNC	0x04
#define TPDAS_A65M_HIRQ_SEL_ADC_IRT_POS		0x05
#define TPDAS_A65M_HIRQ_SEL_ADC_IRT_NEG		0x06
#define TPDAS_A65M_HIRQ_SEL_DAC_IRT_POS		0x07
#define TPDAS_A65M_HIRQ_SEL_DAC_IRT_NEG		0x08
#define TPDAS_A65M_HIRQ_SEL_GPIO0		0x09
#define TPDAS_A65M_HIRQ_SEL_GPIO1		0x0A
#define TPDAS_A65M_HIRQ_SEL_ERR			0x0B
#define TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_FF	0x10	// ADC/DAC FIFO flags valid for HIRQ-0/1 only
#define TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_PAF	0x11
#define TPDAS_A65M_HIRQ__SEL_ADC1_FIFO_EF	0x12
#define TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_PAE	0x13
#define TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_FF	0x14
#define TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_PAF	0x15
#define TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_EF	0x16
#define TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_PAE	0x17
#define TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_FF	0x18
#define TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_PAF	0x19
#define TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_EF	0x1a
#define TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_PAE	0x1b
#define TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_FF	0x1c
#define TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_PAF	0x1d
#define TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_EF	0x1e
#define TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_PAE	0x1f

#define TPDAS_A65M_HIRQ_EN_OFF			0x00	// HIRQ line enable control
#define TPDAS_A65M_HIRQ_EN_ON			0x80

#define TPDAS_A65M_HIRQ_EN_SHIFT		0x7


	//-- HIRQ ERR Aux Interrupt Mask (XIM) Registers bits
#define TPDAS_A65M_XIM_ADC_FIFO_OVF_ON		0x01
#define TPDAS_A65M_XIM_ADC_FIFO_UNF_ON		0x02
#define TPDAS_A65M_XIM_DAC_FIFO_OVF_ON		0x04
#define TPDAS_A65M_XIM_DAC_FIFO_UNF_ON		0x08
#define TPDAS_A65M_XIM_ADC_IRT_ERR_ON		0x10
#define TPDAS_A65M_XIM_DAC_IRT_ERR_ON		0x20
#define TPDAS_A65M_XIM_ADC_FIFO_OVF_OFF		0
#define TPDAS_A65M_XIM_ADC_FIFO_UNF_OFF		0
#define TPDAS_A65M_XIM_DAC_FIFO_OVF_OFF		0
#define TPDAS_A65M_XIM_DAC_FIFO_UNF_OFF		0
#define TPDAS_A65M_XIM_ADC_IRT_ERR_OFF		0
#define TPDAS_A65M_XIM_DAC_IRT_ERR_OFF		0

#define TPDAS_A65M_XIM_ADC_FIFO_OVF_SHIFT	0	// aligned defs
#define TPDAS_A65M_XIM_ADC_FIFO_UNF_SHIFT	1
#define TPDAS_A65M_XIM_DAC_FIFO_OVF_SHIFT	2
#define TPDAS_A65M_XIM_DAC_FIFO_UNF_SHIFT	3
#define TPDAS_A65M_XIM_ADC_IRT_ERR_SHIFT	4
#define TPDAS_A65M_XIM_DAC_IRT_ERR_SHIFT	5

#define TPDAS_A65M_XIM_OFF			0
#define TPDAS_A65M_XIM_ON			1


	//--- GPIO-0..3 defs (DIR and DATA registers) 
#define TPDAS_A65M_GPIO0_DIR_BITMASK		1	// field bitmasks
#define TPDAS_A65M_GPIO1_DIR_BITMASK		2
#define TPDAS_A65M_GPIO2_DIR_BITMASK		4
#define TPDAS_A65M_GPIO3_DIR_BITMASK		8
#define TPDAS_A65M_GPIO0_DATA_BITMASK		1
#define TPDAS_A65M_GPIO1_DATA_BITMASK		2
#define TPDAS_A65M_GPIO2_DATA_BITMASK		4
#define TPDAS_A65M_GPIO3_DATA_BITMASK		8

#define TPDAS_A65M_GPIO0_DIR_INPUT		0
#define TPDAS_A65M_GPIO0_DIR_OUTPUT		1
#define TPDAS_A65M_GPIO1_DIR_INPUT		0
#define TPDAS_A65M_GPIO1_DIR_OUTPUT		2
#define TPDAS_A65M_GPIO2_DIR_INPUT		0
#define TPDAS_A65M_GPIO2_DIR_OUTPUT		4
#define TPDAS_A65M_GPIO3_DIR_INPUT		0
#define TPDAS_A65M_GPIO3_DIR_OUTPUT		8

#define TPDAS_A65M_GPIO0_DATA_OFF		0
#define TPDAS_A65M_GPIO0_DATA_ON		1
#define TPDAS_A65M_GPIO1_DATA_OFF		0
#define TPDAS_A65M_GPIO1_DATA_ON		2
#define TPDAS_A65M_GPIO2_DATA_OFF		0
#define TPDAS_A65M_GPIO2_DATA_ON		4
#define TPDAS_A65M_GPIO3_DATA_OFF		0
#define TPDAS_A65M_GPIO3_DATA_ON		8

#define TPDAS_A65M_GPIO0_DIR_SHIFT		0	// aligned defs
#define TPDAS_A65M_GPIO1_DIR_SHIFT		1
#define TPDAS_A65M_GPIO2_DIR_SHIFT		2
#define TPDAS_A65M_GPIO3_DIR_SHIFT		3
#define TPDAS_A65M_GPIO0_DATA_SHIFT		0
#define TPDAS_A65M_GPIO1_DATA_SHIFT		1
#define TPDAS_A65M_GPIO2_DATA_SHIFT		2
#define TPDAS_A65M_GPIO3_DATA_SHIFT		3

#define TPDAS_A65M_GPIO_DIR_INPUT		0
#define TPDAS_A65M_GPIO_DIR_OUTPUT		1
#define TPDAS_A65M_GPIO_DATA_OFF		0
#define TPDAS_A65M_GPIO_DATA_ON			1


	//--- PFG CNTR1..3 registers defs
#define TPDAS_A65M_PFG_V07_BITMASK		0xff	// PFG CNTR1 register
#define TPDAS_A65M_PFG_V8_BITMASK		0x01	// PFG CNTR2 register
#define TPDAS_A65M_PFG_R_BITMASK		0xfe	// PFG CNTR2 register
#define TPDAS_A65M_PFG_S_BITMASK		0x07	// PFG CNTR3 register
#define TPDAS_A65M_PFG_X_BITMASK		0x10	// PFG CNTR3 register

#define TPDAS_A65M_PFG_R_SHIFT			1

#define TPDAS_A65M_X_DIV1			0	// X-divider field values
#define TPDAS_A65M_X_DIV16			0x10

#define TPDAS_A65M_S_DIV2			1	// S-divider field values
#define TPDAS_A65M_S_DIV4			3
#define TPDAS_A65M_S_DIV5			4
#define TPDAS_A65M_S_DIV6			7
#define TPDAS_A65M_S_DIV7			5
#define TPDAS_A65M_S_DIV8			2
#define TPDAS_A65M_S_DIV9			6
#define TPDAS_A65M_S_DIV10			0

//========================================================================

//============== ADC/DAC FIFO bit/fields definitions =====================

#define TPDAS_A65M_ADDA_DATA_BITMASK		0x0fff	// D0..D11 only are valid for ADC/DAC FIFO data	
#define TPDAS_A65M_XDS_DATA_BITMASK		0xffff	// D0..D15 only are valid for XDSIN/XDSOUT FIFO data	

	//-- read-back ADC FIFO MUX-channel ID (presents for ADC1 FIFO output only)
#define TPDAS_A65M_ADC_RIMUX_BITMASK		0xc000	// field bitmask	
#define TPDAS_A65M_ADC_RIMUX_SHIFT		14	// bit-shift	

#define TPDAS_A65M_ADC_RIMUX_CH0		0x0000	
#define TPDAS_A65M_ADC_RIMUX_CH1		0x4000
#define TPDAS_A65M_ADC_RIMUX_CH2		0x8000
#define TPDAS_A65M_ADC_RIMUX_CH3		0xc000

	//-- ADC/DAC FIFO serial-IN CNF port
#define TPDAS_A65M_FIFO_SIN_BITMASK		0x1	// D0 is only valid for Serial-IN data	

//========================================================================



//========== macros for accessing HIF registers =============

	//~~~ ADC/DAC DAQ controllers reset macros
#define TPDAS_A65M_reset_adc_daq()		(*(unsigned *)TPDAS_A65M_ADC_DAQ_RESET_RG) = 0
#define TPDAS_A65M_reset_dac_daq()		(*(unsigned *)TPDAS_A65M_DAC_DAQ_RESET_RG) = 0
#define TPDAS_A65M_reset_daq()			(*(unsigned *)TPDAS_A65M_DAQ_RESET_RG) = 0

#define TPDAS_A65M_RESET_ADC_DAQ		TPDAS_A65M_reset_adc_daq()
#define TPDAS_A65M_RESET_DAC_DAQ		TPDAS_A65M_reset_dac_daq()
#define TPDAS_A65M_RESET_DAQ			TPDAS_A65M_reset_daq()


	//--- CONTROL-1 register
#define TPDAS_A65M_daq_cntr1_rg_data()		((*(unsigned *)TPDAS_A65M_DAQ_CNTR1_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_daq_cntr1_rg(d)		(*(unsigned *)TPDAS_A65M_DAQ_CNTR1_RG) = d

#define TPDAS_A65M_adc_mode_data()		(TPDAS_A65M_daq_cntr1_rg_data() & TPDAS_A65M_ADC_MODE_BITMASK)
#define TPDAS_A65M_set_adc_mode(d)		TPDAS_A65M_set_daq_cntr1_rg((TPDAS_A65M_daq_cntr1_rg_data() & ~TPDAS_A65M_ADC_MODE_BITMASK) | (d & TPDAS_A65M_ADC_MODE_BITMASK))

#define TPDAS_A65M_dac_mode_data()		(TPDAS_A65M_daq_cntr1_rg_data() & TPDAS_A65M_DAC_MODE_BITMASK)
#define TPDAS_A65M_set_dac_mode(d)		TPDAS_A65M_set_daq_cntr1_rg((TPDAS_A65M_daq_cntr1_rg_data() & ~TPDAS_A65M_DAC_MODE_BITMASK) | (d & TPDAS_A65M_DAC_MODE_BITMASK))

#define TPDAS_A65M_adc_fmt_data()		(TPDAS_A65M_daq_cntr1_rg_data() & TPDAS_A65M_ADC_FMT_BITMASK)
#define TPDAS_A65M_set_adc_fmt(d)		TPDAS_A65M_set_daq_cntr1_rg((TPDAS_A65M_daq_cntr1_rg_data() & ~TPDAS_A65M_ADC_FMT_BITMASK) |  (d & TPDAS_A65M_ADC_FMT_BITMASK))

#define TPDAS_A65M_dac_fmt_data()		(TPDAS_A65M_daq_cntr1_rg_data() & TPDAS_A65M_DAC_FMT_BITMASK)
#define TPDAS_A65M_set_dac_fmt(d)		TPDAS_A65M_set_daq_cntr1_rg((TPDAS_A65M_daq_cntr1_rg_data() & ~TPDAS_A65M_DAC_FMT_BITMASK) |  (d & TPDAS_A65M_DAC_FMT_BITMASK))

		//~~~ setting macros
#define TPDAS_A65M_SET_ADC_MODE_CNF		TPDAS_A65M_set_adc_mode(TPDAS_A65M_ADC_MODE_CNF)
#define TPDAS_A65M_SET_ADC_MODE_OPM		TPDAS_A65M_set_adc_mode(TPDAS_A65M_ADC_MODE_OPM)
#define TPDAS_A65M_SET_ADC_MODE_PTM		TPDAS_A65M_set_adc_mode(TPDAS_A65M_ADC_MODE_PTM)

#define TPDAS_A65M_SET_DAC_MODE_CNF		TPDAS_A65M_set_dac_mode(TPDAS_A65M_DAC_MODE_CNF)
#define TPDAS_A65M_SET_DAC_MODE_OPM		TPDAS_A65M_set_dac_mode(TPDAS_A65M_DAC_MODE_OPM)
#define TPDAS_A65M_SET_DAC_MODE_PTM		TPDAS_A65M_set_dac_mode(TPDAS_A65M_DAC_MODE_PTM)

#define TPDAS_A65M_SET_ADC_FMT_1		TPDAS_A65M_set_adc_fmt(TPDAS_A65M_ADC_FMT_1)
#define TPDAS_A65M_SET_ADC_FMT_2		TPDAS_A65M_set_adc_fmt(TPDAS_A65M_ADC_FMT_2)
#define TPDAS_A65M_SET_ADC_FMT_12		TPDAS_A65M_set_adc_fmt(TPDAS_A65M_ADC_FMT_12)

#define TPDAS_A65M_SET_DAC_FMT_1		TPDAS_A65M_set_dac_fmt(TPDAS_A65M_DAC_FMT_1)
#define TPDAS_A65M_SET_DAC_FMT_2		TPDAS_A65M_set_dac_fmt(TPDAS_A65M_DAC_FMT_2)
#define TPDAS_A65M_SET_DAC_FMT_12		TPDAS_A65M_set_dac_fmt(TPDAS_A65M_DAC_FMT_12)

		//~~~ macros for data aligned at bit #0
		//    input p/arams: 
		//    -------------	
		//	MODE: 	TPDAS_A65M_ADDA_MODE_CNF,TPDAS_A65M_ADDA_MODE_PTM,TPDAS_A65M_ADDA_MODE_OPM
		//	FORMAT:	TPDAS_A65M_ADDA_FMT_1,TPDAS_A65M_ADDA_FMT_2,TPDAS_A65M_ADDA_FMT_12
#define TPDAS_A65M_ADC_MODE_DATA		TPDAS_A65M_adc_mode_data()
#define TPDAS_A65M_SET_ADC_MODE(d)		TPDAS_A65M_set_adc_mode(d)

#define TPDAS_A65M_DAC_MODE_DATA		(TPDAS_A65M_dac_mode_data() >>TPDAS_A65M_DAC_MODE_SHIFT)
#define TPDAS_A65M_SET_DAC_MODE(d)		TPDAS_A65M_set_dac_mode(d<<TPDAS_A65M_DAC_MODE_SHIFT)

#define TPDAS_A65M_ADC_FMT_DATA			(TPDAS_A65M_adc_fmt_data() >>TPDAS_A65M_ADC_FMT_SHIFT)
#define TPDAS_A65M_SET_ADC_FMT(d)		TPDAS_A65M_set_adc_fmt(d<<TPDAS_A65M_ADC_FMT_SHIFT)

#define TPDAS_A65M_DAC_FMT_DATA			(TPDAS_A65M_dac_fmt_data() >> TPDAS_A65M_DAC_FMT_SHIFT)
#define TPDAS_A65M_SET_DAC_FMT(d)		TPDAS_A65M_set_dac_fmt(d<<TPDAS_A65M_DAC_FMT_SHIFT)


	//--- CONTROL-2 register
#define TPDAS_A65M_daq_cntr2_rg_data()		((*(unsigned *)TPDAS_A65M_DAQ_CNTR2_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_daq_cntr2_rg(d)		(*(unsigned *)TPDAS_A65M_DAQ_CNTR2_RG) = d

#define TPDAS_A65M_adc_daq_run_data()		(TPDAS_A65M_daq_cntr2_rg_data() & TPDAS_A65M_ADC_DAQ_RUN_BITMASK)
#define TPDAS_A65M_set_adc_daq_run()		TPDAS_A65M_set_daq_cntr2_rg(TPDAS_A65M_ADC_DAQ_RUN)

#define TPDAS_A65M_dac_daq_run_data()		(TPDAS_A65M_daq_cntr2_rg_data() & TPDAS_A65M_DAC_DAQ_RUN_BITMASK)
#define TPDAS_A65M_set_dac_daq_run()		TPDAS_A65M_set_daq_cntr2_rg(TPDAS_A65M_DAC_DAQ_RUN)

#define TPDAS_A65M_set_adc_daq_abort()		TPDAS_A65M_set_daq_cntr2_rg(TPDAS_A65M_ADC_DAQ_ABORT)
#define TPDAS_A65M_set_dac_daq_abort()		TPDAS_A65M_set_daq_cntr2_rg(TPDAS_A65M_DAC_DAQ_ABORT)

#define TPDAS_A65M_adc_daq_end_data()		(TPDAS_A65M_daq_cntr2_rg_data() & TPDAS_A65M_ADC_DAQ_END_BITMASK)
#define TPDAS_A65M_dac_daq_end_data()		(TPDAS_A65M_daq_cntr2_rg_data() & TPDAS_A65M_DAC_DAQ_END_BITMASK)
#define TPDAS_A65M_adc_daq_sync_data()		(TPDAS_A65M_daq_cntr2_rg_data() & TPDAS_A65M_ADC_DAQ_SYNC_BITMASK)
#define TPDAS_A65M_dac_daq_sync_data()		(TPDAS_A65M_daq_cntr2_rg_data() & TPDAS_A65M_DAC_DAQ_SYNC_BITMASK)

		//~~~ setting macros
#define TPDAS_A65M_SET_ADC_DAQ_RUN		TPDAS_A65M_set_adc_daq_run()
#define TPDAS_A65M_SET_DAC_DAQ_RUN		TPDAS_A65M_set_dac_daq_run()
#define TPDAS_A65M_SET_ADC_DAQ_ABORT		TPDAS_A65M_set_adc_daq_abort()
#define TPDAS_A65M_SET_DAC_DAQ_ABORT		TPDAS_A65M_set_dac_daq_abort()


	//--- CONTROL-3 register
#define TPDAS_A65M_daq_cntr3_rg_data()		((*(unsigned *)TPDAS_A65M_DAQ_CNTR3_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_daq_cntr3_rg(d)		(*(unsigned *)TPDAS_A65M_DAQ_CNTR3_RG) = d

#define TPDAS_A65M_adc_daq_tf_sel_data()	(TPDAS_A65M_daq_cntr3_rg_data() & TPDAS_A65M_ADC_DAQ_TF_SEL_BITMASK)
#define TPDAS_A65M_set_adc_daq_tf_sel(d)	TPDAS_A65M_set_daq_cntr3_rg((TPDAS_A65M_daq_cntr3_rg_data() & ~TPDAS_A65M_ADC_DAQ_TF_SEL_BITMASK) |  (d & TPDAS_A65M_ADC_DAQ_TF_SEL_BITMASK))

#define TPDAS_A65M_dac_daq_tf_sel_data()	(TPDAS_A65M_daq_cntr3_rg_data() & TPDAS_A65M_DAC_DAQ_TF_SEL_BITMASK)
#define TPDAS_A65M_set_dac_daq_tf_sel(d)	TPDAS_A65M_set_daq_cntr3_rg((TPDAS_A65M_daq_cntr3_rg_data() & ~TPDAS_A65M_DAC_DAQ_TF_SEL_BITMASK) |  (d & TPDAS_A65M_DAC_DAQ_TF_SEL_BITMASK))

#define TPDAS_A65M_adc_irt_tf_sel_data()	(TPDAS_A65M_daq_cntr3_rg_data() & TPDAS_A65M_ADC_IRT_TF_SEL_BITMASK)
#define TPDAS_A65M_set_adc_irt_tf_sel(d)	TPDAS_A65M_set_daq_cntr3_rg((TPDAS_A65M_daq_cntr3_rg_data() & ~TPDAS_A65M_ADC_IRT_TF_SEL_BITMASK) |  (d & TPDAS_A65M_ADC_IRT_TF_SEL_BITMASK))

#define TPDAS_A65M_dac_irt_tf_sel_data()	(TPDAS_A65M_daq_cntr3_rg_data() & TPDAS_A65M_DAC_IRT_TF_SEL_BITMASK)
#define TPDAS_A65M_set_dac_irt_tf_sel(d)	TPDAS_A65M_set_daq_cntr3_rg((TPDAS_A65M_daq_cntr3_rg_data() & ~TPDAS_A65M_DAC_IRT_TF_SEL_BITMASK) |  (d & TPDAS_A65M_DAC_IRT_TF_SEL_BITMASK))

		//~~~ setting macros
#define TPDAS_A65M_SET_ADC_DAQ_TF_SEL_FF	TPDAS_A65M_set_adc_daq_tf_sel(TPDAS_A65M_ADC_DAQ_TF_SEL_FF)
#define TPDAS_A65M_SET_ADC_DAQ_TF_SEL_PAF	TPDAS_A65M_set_adc_daq_tf_sel(TPDAS_A65M_ADC_DAQ_TF_SEL_PAF)
#define TPDAS_A65M_SET_DAC_DAQ_TF_SEL_EF	TPDAS_A65M_set_dac_daq_tf_sel(TPDAS_A65M_DAC_DAQ_TF_SEL_EF)
#define TPDAS_A65M_SET_DAC_DAQ_TF_SEL_PAE	TPDAS_A65M_set_dac_daq_tf_sel(TPDAS_A65M_DAC_DAQ_TF_SEL_PAE)
#define TPDAS_A65M_SET_ADC_IRT_TF_SEL_EF	TPDAS_A65M_set_adc_irt_tf_sel(TPDAS_A65M_ADC_IRT_TF_SEL_EF)
#define TPDAS_A65M_SET_ADC_IRT_TF_SEL_PAE	TPDAS_A65M_set_adc_irt_tf_sel(TPDAS_A65M_ADC_IRT_TF_SEL_PAE)
#define TPDAS_A65M_SET_DAC_IRT_TF_SEL_FF	TPDAS_A65M_set_dac_irt_tf_sel(TPDAS_A65M_DAC_IRT_TF_SEL_FF)
#define TPDAS_A65M_SET_DAC_IRT_TF_SEL_PAF	TPDAS_A65M_set_dac_irt_tf_sel(TPDAS_A65M_DAC_IRT_TF_SEL_PAF)


	//-- DAQ SYNC Register
#define TPDAS_A65M_daq_sync_rg_data()		((*(unsigned *)TPDAS_A65M_DAQ_SYNC_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_daq_sync_rg(d)		(*(unsigned *)TPDAS_A65M_DAQ_SYNC_RG) = d

#define TPDAS_A65M_fs_sel_data()		(TPDAS_A65M_daq_sync_rg_data() & TPDAS_A65M_FS_SEL_BITMASK)
#define TPDAS_A65M_set_fs_sel(d)		TPDAS_A65M_set_daq_sync_rg((TPDAS_A65M_daq_sync_rg_data() & ~TPDAS_A65M_FS_SEL_BITMASK) |  (d & TPDAS_A65M_FS_SEL_BITMASK))

#define TPDAS_A65M_adc_sync_sel_data()		(TPDAS_A65M_daq_sync_rg_data() & TPDAS_A65M_ADC_SYNC_SEL_BITMASK)
#define TPDAS_A65M_set_adc_sync_sel(d)		TPDAS_A65M_set_daq_sync_rg((TPDAS_A65M_daq_sync_rg_data() & ~TPDAS_A65M_ADC_SYNC_SEL_BITMASK) |  (d & TPDAS_A65M_ADC_SYNC_SEL_BITMASK))

#define TPDAS_A65M_dac_sync_sel_data()		(TPDAS_A65M_daq_sync_rg_data() & TPDAS_A65M_DAC_SYNC_SEL_BITMASK)
#define TPDAS_A65M_set_dac_sync_sel(d)		TPDAS_A65M_set_daq_sync_rg((TPDAS_A65M_daq_sync_rg_data() & ~TPDAS_A65M_DAC_SYNC_SEL_BITMASK) |  (d & TPDAS_A65M_DAC_SYNC_SEL_BITMASK))

#define TPDAS_A65M_xsync_mode_data()		(TPDAS_A65M_daq_sync_rg_data() & TPDAS_A65M_XSYNC_MODE_BITMASK)
#define TPDAS_A65M_set_xsync_mode(d)		TPDAS_A65M_set_daq_sync_rg((TPDAS_A65M_daq_sync_rg_data() & ~TPDAS_A65M_XSYNC_MODE_BITMASK) |  (d & TPDAS_A65M_XSYNC_MODE_BITMASK))

#define TPDAS_A65M_sync_mode_data()		(TPDAS_A65M_daq_sync_rg_data() & TPDAS_A65M_SYNC_MODE_BITMASK)
#define TPDAS_A65M_set_sync_mode(d)		TPDAS_A65M_set_daq_sync_rg((TPDAS_A65M_daq_sync_rg_data() & ~TPDAS_A65M_SYNC_MODE_BITMASK) |  (d & TPDAS_A65M_SYNC_MODE_BITMASK))

		//~~~ setting macros
#define TPDAS_A65M_SET_FS_SEL_PFG		TPDAS_A65M_set_fs_sel(TPDAS_A65M_FS_SEL_PFG)
#define TPDAS_A65M_SET_FS_SEL_XFS		TPDAS_A65M_set_fs_sel(TPDAS_A65M_FS_SEL_XFS)
#define TPDAS_A65M_SET_FS_SEL_PXTM0		TPDAS_A65M_set_fs_sel(TPDAS_A65M_FS_SEL_PXTM0)
#define TPDAS_A65M_SET_FS_SEL_PXTM1		TPDAS_A65M_set_fs_sel(TPDAS_A65M_FS_SEL_PXTM1)
#define TPDAS_A65M_SET_ADC_SYNC_SEL_SFT		TPDAS_A65M_set_adc_sync_sel(TPDAS_A65M_ADC_SYNC_SEL_SFT)
#define TPDAS_A65M_SET_ADC_SYNC_SEL_XSYNC	TPDAS_A65M_set_adc_sync_sel(TPDAS_A65M_ADC_SYNC_SEL_XSYNC)
#define TPDAS_A65M_SET_DAC_SYNC_SEL_SFT		TPDAS_A65M_set_dac_sync_sel(TPDAS_A65M_DAC_SYNC_SEL_SFT)
#define TPDAS_A65M_SET_DAC_SYNC_SEL_XSYNC	TPDAS_A65M_set_dac_sync_sel(TPDAS_A65M_DAC_SYNC_SEL_XSYNC)
#define TPDAS_A65M_SET_XSYNC_MODE_LEVEL		TPDAS_A65M_set_xsync_mode(TPDAS_A65M_XSYNC_MODE_LEVEL)
#define TPDAS_A65M_SET_XSYNC_MODE_EDGE		TPDAS_A65M_set_xsync_mode(TPDAS_A65M_XSYNC_MODE_EDGE)
#define TPDAS_A65M_SET_SYNC_MODE_MASTER		TPDAS_A65M_set_xsync_mode(TPDAS_A65M_SYNC_MODE_MASTER)
#define TPDAS_A65M_SET_SYNC_MODE_SLAVE		TPDAS_A65M_set_xsync_mode(TPDAS_A65M_SYNC_MODE_SLAVE)

		//~~~ macros for data aligned at bit #0
		//    input params: 
		//    -------------	
		//	SYNC: 	TPDAS_A65M_ADDA_SYNC_SEL_SFT,TPDAS_A65M_ADDA_SYNC_SEL_XSYNC
#define TPDAS_A65M_ADC_SYNC_SEL_DATA		(TPDAS_A65M_adc_sync_sel_data()	>>TPDAS_A65M_ADC_SYNC_SEL_SHIFT)
#define TPDAS_A65M_SET_ADC_SYNC_SEL(d)		TPDAS_A65M_set_adc_sync_sel(d <<TPDAS_A65M_ADC_SYNC_SEL_SHIFT)
#define TPDAS_A65M_DAC_SYNC_SEL_DATA		(TPDAS_A65M_dac_sync_sel_data() >>TPDAS_A65M_DAC_SYNC_SEL_SHIFT)
#define TPDAS_A65M_SET_DAC_SYNC_SEL(d)		TPDAS_A65M_set_dac_sync_sel(d <<TPDAS_A65M_DAC_SYNC_SEL_SHIFT)



	//-- DAQ MUX Register
#define TPDAS_A65M_daq_mux_rg_data()		((*(unsigned *)TPDAS_A65M_DAQ_MUX_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_daq_mux_rg(d)		(*(unsigned *)TPDAS_A65M_DAQ_MUX_RG) = d

#define TPDAS_A65M_adc_imux_data()		(TPDAS_A65M_daq_mux_rg_data() & TPDAS_A65M_ADC_IMUX_BITMASK)
#define TPDAS_A65M_set_adc_imux(d)		TPDAS_A65M_set_daq_mux_rg((TPDAS_A65M_daq_mux_rg_data() & ~TPDAS_A65M_ADC_IMUX_BITMASK) |  (d & TPDAS_A65M_ADC_IMUX_BITMASK))

#define TPDAS_A65M_dsin_sel_data()		(TPDAS_A65M_daq_mux_rg_data() & TPDAS_A65M_DSIN_SEL_BITMASK)
#define TPDAS_A65M_set_dsin_sel(d)		TPDAS_A65M_set_daq_mux_rg((TPDAS_A65M_daq_mux_rg_data() & ~TPDAS_A65M_DSIN_SEL_BITMASK) |  (d & TPDAS_A65M_DSIN_SEL_BITMASK))

#define TPDAS_A65M_dsout_sel_data()		(TPDAS_A65M_daq_mux_rg_data() & TPDAS_A65M_DSOUT_SEL_BITMASK)
#define TPDAS_A65M_set_dsout_sel(d)		TPDAS_A65M_set_daq_mux_rg((TPDAS_A65M_daq_mux_rg_data() & ~TPDAS_A65M_DSOUT_SEL_BITMASK) |  (d & TPDAS_A65M_DSOUT_SEL_BITMASK))

		//~~~ setting macros
#define TPDAS_A65M_SET_ADC_IMUX_CH0		TPDAS_A65M_set_adc_imux(TPDAS_A65M_ADC_IMUX_CH0)
#define TPDAS_A65M_SET_ADC_IMUX_CH1		TPDAS_A65M_set_adc_imux(TPDAS_A65M_ADC_IMUX_CH1)
#define TPDAS_A65M_SET_ADC_IMUX_CH2		TPDAS_A65M_set_adc_imux(TPDAS_A65M_ADC_IMUX_CH2)
#define TPDAS_A65M_SET_ADC_IMUX_CH3		TPDAS_A65M_set_adc_imux(TPDAS_A65M_ADC_IMUX_CH3)
#define TPDAS_A65M_SET_DSIN_SEL_ADC		TPDAS_A65M_set_dsin_sel(TPDAS_A65M_DSIN_SEL_ADC)
#define TPDAS_A65M_SET_DSIN_SEL_XDSIN		TPDAS_A65M_set_dsin_sel(TPDAS_A65M_DSIN_SEL_XDSIN)
#define TPDAS_A65M_SET_DSOUT_SEL_DAC		TPDAS_A65M_set_dsout_sel(TPDAS_A65M_DSOUT_SEL_DAC)
#define TPDAS_A65M_SET_DSOUT_SEL_XDSOUT		TPDAS_A65M_set_dsout_sel(TPDAS_A65M_DSOUT_SEL_XDSOUT)


	//-- ADC/DAC FIFO status Register
#define TPDAS_A65M_adc_fifo_stat_rg_data()	((*(unsigned *)TPDAS_A65M_ADC_FIFO_STAT_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_dac_fifo_stat_rg_data()	((*(unsigned *)TPDAS_A65M_DAC_FIFO_STAT_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)

#define TPDAS_A65M_adc1_fifo_ff_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC1_FIFO_FF_BITMASK)
#define TPDAS_A65M_adc1_fifo_paf_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC1_FIFO_PAF_BITMASK)
#define TPDAS_A65M_adc1_fifo_ef_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC1_FIFO_EF_BITMASK)
#define TPDAS_A65M_adc1_fifo_pae_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC1_FIFO_PAE_BITMASK)
#define TPDAS_A65M_adc2_fifo_ff_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC2_FIFO_FF_BITMASK)
#define TPDAS_A65M_adc2_fifo_paf_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC2_FIFO_PAF_BITMASK)
#define TPDAS_A65M_adc2_fifo_ef_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC2_FIFO_EF_BITMASK)
#define TPDAS_A65M_adc2_fifo_pae_data()		(TPDAS_A65M_adc_fifo_stat_rg_data() & TPDAS_A65M_ADC2_FIFO_PAE_BITMASK)

#define TPDAS_A65M_dac1_fifo_ff_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC1_FIFO_FF_BITMASK)
#define TPDAS_A65M_dac1_fifo_paf_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC1_FIFO_PAF_BITMASK)
#define TPDAS_A65M_dac1_fifo_ef_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC1_FIFO_EF_BITMASK)
#define TPDAS_A65M_dac1_fifo_pae_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC1_FIFO_PAE_BITMASK)
#define TPDAS_A65M_dac2_fifo_ff_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC2_FIFO_FF_BITMASK)
#define TPDAS_A65M_dac2_fifo_paf_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC2_FIFO_PAF_BITMASK)
#define TPDAS_A65M_dac2_fifo_ef_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC2_FIFO_EF_BITMASK)
#define TPDAS_A65M_dac2_fifo_pae_data()		(TPDAS_A65M_dac_fifo_stat_rg_data() & TPDAS_A65M_DAC2_FIFO_PAE_BITMASK)

		//~~~ data returning macros with data aligned at bit #0
		//    (can be used to provide AND/OR logical functions over FIFO status info)
#define TPDAS_A65M_ADC1_FIFO_FF_DATA		TPDAS_A65M_adc1_fifo_ff_data()
#define TPDAS_A65M_ADC1_FIFO_PAF_DATA		(TPDAS_A65M_adc1_fifo_paf_data() >> TPDAS_A65M_ADDA1_FIFO_PAF_SHIFT)
#define TPDAS_A65M_ADC1_FIFO_EF_DATA		(TPDAS_A65M_adc1_fifo_ef_data()  >> TPDAS_A65M_ADDA1_FIFO_EF_SHIFT)
#define TPDAS_A65M_ADC1_FIFO_PAE_DATA		(TPDAS_A65M_adc1_fifo_pae_data() >> TPDAS_A65M_ADDA1_FIFO_PAE_SHIFT)
#define TPDAS_A65M_ADC2_FIFO_FF_DATA		(TPDAS_A65M_adc2_fifo_ff_data()  >> TPDAS_A65M_ADDA2_FIFO_FF_SHIFT)
#define TPDAS_A65M_ADC2_FIFO_PAF_DATA		(TPDAS_A65M_adc2_fifo_paf_data() >> TPDAS_A65M_ADDA2_FIFO_PAF_SHIFT)
#define TPDAS_A65M_ADC2_FIFO_EF_DATA		(TPDAS_A65M_adc2_fifo_ef_data()  >> TPDAS_A65M_ADDA2_FIFO_EF_SHIFT)
#define TPDAS_A65M_ADC2_FIFO_PAE_DATA		(TPDAS_A65M_adc2_fifo_pae_data() >> TPDAS_A65M_ADDA2_FIFO_PAE_SHIFT)

#define TPDAS_A65M_DAC1_FIFO_FF_DATA		(TPDAS_A65M_dac1_fifo_ff_data() 
#define TPDAS_A65M_DAC1_FIFO_PAF_DATA		(TPDAS_A65M_dac1_fifo_paf_data() >> TPDAS_A65M_ADDA1_FIFO_PAF_SHIFT)
#define TPDAS_A65M_DAC1_FIFO_EF_DATA		(TPDAS_A65M_dac1_fifo_ef_data()	 >> TPDAS_A65M_ADDA1_FIFO_EF_SHIFT)
#define TPDAS_A65M_DAC1_FIFO_PAE_DATA		(TPDAS_A65M_dac1_fifo_pae_data() >> TPDAS_A65M_ADDA1_FIFO_PAE_SHIFT)
#define TPDAS_A65M_DAC2_FIFO_FF_DATA		(TPDAS_A65M_dac2_fifo_ff_data()  >> TPDAS_A65M_ADDA2_FIFO_FF_SHIFT)
#define TPDAS_A65M_DAC2_FIFO_PAF_DATA		(TPDAS_A65M_dac2_fifo_paf_data() >> TPDAS_A65M_ADDA2_FIFO_PAF_SHIFT)
#define TPDAS_A65M_DAC2_FIFO_EF_DATA		(TPDAS_A65M_dac2_fifo_ef_data()  >> TPDAS_A65M_ADDA2_FIFO_EF_SHIFT)
#define TPDAS_A65M_DAC2_FIFO_PAE_DATA		(TPDAS_A65M_dac2_fifo_pae_data() >> TPDAS_A65M_ADDA2_FIFO_PAE_SHIFT)



	//-- ERROR status/clear Registers
#define TPDAS_A65M_err_stat_rg_data()		((*(unsigned *)TPDAS_A65M_ERR_STAT_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_err_clr_rg(d)		(*(unsigned *)TPDAS_A65M_ERR_CLR_RG) = d

#define TPDAS_A65M_adc_fifo_ovf_data()		(TPDAS_A65M_err_stat_rg_data() & TPDAS_A65M_ADC_FIFO_OVF_BITMASK)
#define TPDAS_A65M_adc_fifo_unf_data()		(TPDAS_A65M_err_stat_rg_data() & TPDAS_A65M_ADC_FIFO_UNF_BITMASK)
#define TPDAS_A65M_dac_fifo_ovf_data()		(TPDAS_A65M_err_stat_rg_data() & TPDAS_A65M_DAC_FIFO_OVF_BITMASK)
#define TPDAS_A65M_dac_fifo_unf_data()		(TPDAS_A65M_err_stat_rg_data() & TPDAS_A65M_DAC_FIFO_UNF_BITMASK)
#define TPDAS_A65M_adc_irt_err_data()		(TPDAS_A65M_err_stat_rg_data() & TPDAS_A65M_ADC_IRT_ERR_BITMASK)
#define TPDAS_A65M_dac_irt_err_data()		(TPDAS_A65M_err_stat_rg_data() & TPDAS_A65M_DAC_IRT_ERR_BITMASK)

#define TPDAS_A65M_clr_adc_fifo_ovf()		TPDAS_A65M_set_err_clr_rg(TPDAS_A65M_ADC_FIFO_OVF_CLR)
#define TPDAS_A65M_clr_adc_fifo_unf()		TPDAS_A65M_set_err_clr_rg(TPDAS_A65M_ADC_FIFO_UNF_CLR)
#define TPDAS_A65M_clr_dac_fifo_ovf()		TPDAS_A65M_set_err_clr_rg(TPDAS_A65M_DAC_FIFO_OVF_CLR)
#define TPDAS_A65M_clr_dac_fifo_unf()		TPDAS_A65M_set_err_clr_rg(TPDAS_A65M_DAC_FIFO_UNF_CLR)
#define TPDAS_A65M_clr_adc_irt_err()		TPDAS_A65M_set_err_clr_rg(TPDAS_A65M_ADC_IRT_ERR_CLR)
#define TPDAS_A65M_clr_dac_irt_err()		TPDAS_A65M_set_err_clr_rg(TPDAS_A65M_DAC_IRT_ERR_CLR)

		//~~~ setting macros
#define TPDAS_A65M_CLR_ERR_ALL			TPDAS_A65M_set_err_clr_rg(0xff)
#define TPDAS_A65M_CLR_ADC_FIFO_OVF		TPDAS_A65M_clr_adc_fifo_ovf()
#define TPDAS_A65M_CLR_ADC_FIFO_UNF		TPDAS_A65M_clr_adc_fifo_unf()
#define TPDAS_A65M_CLR_DAC_FIFO_OVF		TPDAS_A65M_clr_dac_fifo_ovf()
#define TPDAS_A65M_CLR_DAC_FIFO_UNF		TPDAS_A65M_clr_dac_fifo_unf()
#define TPDAS_A65M_CLR_ADC_IRT_ERR		TPDAS_A65M_clr_adc_irt_err()
#define TPDAS_A65M_CLR_DAC_IRT_ERR		TPDAS_A65M_clr_dac_irt_err()

		//~~~ data returning macros with data aligned at bit #0
		//    (can be used to provide AND/OR logical functions over ERROR status info)
#define TPDAS_A65M_ADC_FIFO_OVF_DATA		TPDAS_A65M_adc_fifo_ovf_data()
#define TPDAS_A65M_ADC_FIFO_UNF_DATA		(TPDAS_A65M_adc_fifo_unf_data() >> TPDAS_A65M_ADC_FIFO_UNF_SHIFT)
#define TPDAS_A65M_DAC_FIFO_OVF_DATA		(TPDAS_A65M_dac_fifo_ovf_data() >> TPDAS_A65M_DAC_FIFO_OVF_SHIFT)
#define TPDAS_A65M_DAC_FIFO_UNF_DATA		(TPDAS_A65M_dac_fifo_unf_data() >> TPDAS_A65M_DAC_FIFO_UNF_SHIFT)
#define TPDAS_A65M_ADC_IRT_ERR_DATA		(TPDAS_A65M_adc_irt_err_data()  >> TPDAS_A65M_ADC_IRT_ERR_SHIFT)
#define TPDAS_A65M_DAC_IRT_ERR_DATA		(TPDAS_A65M_dac_irt_err_data()  >> TPDAS_A65M_DAC_IRT_ERR_SHIFT)



	//-- HIRQ-0 Interrupt control Register
#define TPDAS_A65M_hirq0_sel_rg_data()		((*(unsigned *)TPDAS_A65M_HIRQ0_SEL_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_hirq0_sel_rg(d)		(*(unsigned *)TPDAS_A65M_HIRQ0_SEL_RG) = d

#define TPDAS_A65M_hirq0_sel_data()		(TPDAS_A65M_hirq0_sel_rg_data() & TPDAS_A65M_HIRQ_SEL_BITMASK)
#define TPDAS_A65M_set_hirq0_sel(d)		TPDAS_A65M_set_hirq0_sel_rg((TPDAS_A65M_hirq0_sel_rg_data() & ~TPDAS_A65M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A65M_HIRQ_SEL_BITMASK))
#define TPDAS_A65M_hirq0_en_data()		(TPDAS_A65M_hirq0_sel_rg_data() & TPDAS_A65M_HIRQ_EN_BITMASK)
#define TPDAS_A65M_set_hirq0_en(d)		TPDAS_A65M_set_hirq0_sel_rg((TPDAS_A65M_hirq0_sel_rg_data() & ~TPDAS_A65M_HIRQ_EN_BITMASK) |  (d & TPDAS_A65M_HIRQ_EN_BITMASK))

#define TPDAS_A65M_HIRQ0_SEL_RG_DATA		TPDAS_A65M_hirq0_sel_rg_data()
#define TPDAS_A65M_SET_HIRQ0_SEL_RG(d)		TPDAS_A65M_set_hirq0_sel_rg(d)
#define TPDAS_A65M_HIRQ0_SEL_DATA		TPDAS_A65M_hirq0_sel_data()
#define TPDAS_A65M_SET_HIRQ0_SEL(d)		TPDAS_A65M_set_hirq0_sel(d)
#define TPDAS_A65M_HIRQ0_EN_DATA		TPDAS_A65M_hirq0_en_data()
#define TPDAS_A65M_SET_HIRQ0_EN(d)		TPDAS_A65M_set_hirq0_en(d)

		//~~~ setting macros
#define TPDAS_A65M_SET_HIRQ0_OFF		TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_NO | TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ0_ADC_DAQ_END	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC_DAQ_END	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC_DAQ_SYNC	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC_DAQ_SYNC	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC_IRT_POS	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC_IRT_POS	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC_IRT_NEG	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC_IRT_NEG	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_GPIO0		TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO0        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_GPIO1		TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO1        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ERR		TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ERR          | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC1_FIFO_FF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC1_FIFO_PAF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC1_FIFO_EF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC1_FIFO_PAE	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC2_FIFO_FF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC2_FIFO_PAF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC2_FIFO_EF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_ADC2_FIFO_PAE	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC1_FIFO_FF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC1_FIFO_PAF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC1_FIFO_EF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC1_FIFO_PAE	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC2_FIFO_FF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC2_FIFO_PAF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC2_FIFO_EF	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ0_DAC2_FIFO_PAE	TPDAS_A65M_set_hirq0_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)

#define TPDAS_A65M_SET_HIRQ0_SEL_OFF		TPDAS_A65M_set_hirq0_sel(TPDAS_A65M_HIRQ_SEL_NO)
#define TPDAS_A65M_SET_HIRQ0_EN_OFF		TPDAS_A65M_set_hirq0_en(TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ0_EN_ON		TPDAS_A65M_set_hirq0_en(TPDAS_A65M_HIRQ_EN_ON)


	//-- HIRQ-1 Interrupt control Register
#define TPDAS_A65M_hirq1_sel_rg_data()		((*(unsigned *)TPDAS_A65M_HIRQ1_SEL_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_hirq1_sel_rg(d)		(*(unsigned *)TPDAS_A65M_HIRQ1_SEL_RG) = d

#define TPDAS_A65M_hirq1_sel_data()		(TPDAS_A65M_hirq1_sel_rg_data() & TPDAS_A65M_HIRQ_SEL_BITMASK)
#define TPDAS_A65M_set_hirq1_sel(d)		TPDAS_A65M_set_hirq1_sel_rg((TPDAS_A65M_hirq1_sel_rg_data() & ~TPDAS_A65M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A65M_HIRQ_SEL_BITMASK))
#define TPDAS_A65M_hirq1_en_data()		(TPDAS_A65M_hirq1_sel_rg_data() & TPDAS_A65M_HIRQ_EN_BITMASK)
#define TPDAS_A65M_set_hirq1_en(d)		TPDAS_A65M_set_hirq1_sel_rg((TPDAS_A65M_hirq1_sel_rg_data() & ~TPDAS_A65M_HIRQ_EN_BITMASK) |  (d & TPDAS_A65M_HIRQ_EN_BITMASK))

#define TPDAS_A65M_HIRQ1_SEL_RG_DATA		TPDAS_A65M_hirq1_sel_rg_data()
#define TPDAS_A65M_SET_HIRQ1_SEL_RG(d)		TPDAS_A65M_set_hirq1_sel_rg(d)
#define TPDAS_A65M_HIRQ1_SEL_DATA		TPDAS_A65M_hirq1_sel_data()
#define TPDAS_A65M_SET_HIRQ1_SEL(d)		TPDAS_A65M_set_hirq1_sel(d)
#define TPDAS_A65M_HIRQ1_EN_DATA		TPDAS_A65M_hirq1_en_data()
#define TPDAS_A65M_SET_HIRQ1_EN(d)		TPDAS_A65M_set_hirq1_en(d)

		//~~~ setting macros
#define TPDAS_A65M_SET_HIRQ1_OFF		TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_NO | TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ1_ADC_DAQ_END	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC_DAQ_END	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC_DAQ_SYNC	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC_DAQ_SYNC	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC_IRT_POS	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC_IRT_POS	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC_IRT_NEG	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC_IRT_NEG	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_GPIO0		TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO0        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_GPIO1		TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO1        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ERR		TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ERR          | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC1_FIFO_FF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC1_FIFO_PAF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC1_FIFO_EF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC1_FIFO_PAE	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC1_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC2_FIFO_FF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC2_FIFO_PAF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC2_FIFO_EF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_ADC2_FIFO_PAE	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC2_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC1_FIFO_FF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC1_FIFO_PAF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC1_FIFO_EF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC1_FIFO_PAE	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC1_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC2_FIFO_FF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_FF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC2_FIFO_PAF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_PAF| TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC2_FIFO_EF	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_EF | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ1_DAC2_FIFO_PAE	TPDAS_A65M_set_hirq1_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC2_FIFO_PAE| TPDAS_A65M_HIRQ_EN_ON)

#define TPDAS_A65M_SET_HIRQ1_SEL_OFF		TPDAS_A65M_set_hirq1_sel(TPDAS_A65M_HIRQ_SEL_NO)
#define TPDAS_A65M_SET_HIRQ1_EN_OFF		TPDAS_A65M_set_hirq1_en(TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ1_EN_ON		TPDAS_A65M_set_hirq1_en(TPDAS_A65M_HIRQ_EN_ON)


	//-- HIRQ-2 Interrupt control Register
#define TPDAS_A65M_hirq2_sel_rg_data()		((*(unsigned *)TPDAS_A65M_HIRQ2_SEL_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_hirq2_sel_rg(d)		(*(unsigned *)TPDAS_A65M_HIRQ2_SEL_RG) = d

#define TPDAS_A65M_hirq2_sel_data()		(TPDAS_A65M_hirq2_sel_rg_data() & TPDAS_A65M_HIRQ_SEL_BITMASK)
#define TPDAS_A65M_set_hirq2_sel(d)		TPDAS_A65M_set_hirq2_sel_rg((TPDAS_A65M_hirq2_sel_rg_data() & ~TPDAS_A65M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A65M_HIRQ_SEL_BITMASK))
#define TPDAS_A65M_hirq2_en_data()		(TPDAS_A65M_hirq2_sel_rg_data() & TPDAS_A65M_HIRQ_EN_BITMASK)
#define TPDAS_A65M_set_hirq2_en(d)		TPDAS_A65M_set_hirq2_sel_rg((TPDAS_A65M_hirq2_sel_rg_data() & ~TPDAS_A65M_HIRQ_EN_BITMASK) |  (d & TPDAS_A65M_HIRQ_EN_BITMASK))

#define TPDAS_A65M_HIRQ2_SEL_RG_DATA		TPDAS_A65M_hirq2_sel_rg_data()
#define TPDAS_A65M_SET_HIRQ2_SEL_RG(d)		TPDAS_A65M_set_hirq2_sel_rg(d)
#define TPDAS_A65M_HIRQ2_SEL_DATA		TPDAS_A65M_hirq2_sel_data()
#define TPDAS_A65M_SET_HIRQ2_SEL(d)		TPDAS_A65M_set_hirq2_sel(d)
#define TPDAS_A65M_HIRQ2_EN_DATA		TPDAS_A65M_hirq2_en_data()
#define TPDAS_A65M_SET_HIRQ2_EN(d)		TPDAS_A65M_set_hirq2_en(d)

		//~~~ setting macros
#define TPDAS_A65M_SET_HIRQ2_OFF		TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_NO | TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ2_ADC_DAQ_END	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_DAC_DAQ_END	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_ADC_DAQ_SYNC	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_DAC_DAQ_SYNC	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_ADC_IRT_POS	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_DAC_IRT_POS	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_ADC_IRT_NEG	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_DAC_IRT_NEG	TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_GPIO0		TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO0        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_GPIO1		TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO1        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ2_ERR		TPDAS_A65M_set_hirq2_sel_rg(TPDAS_A65M_HIRQ_SEL_ERR          | TPDAS_A65M_HIRQ_EN_ON)

#define TPDAS_A65M_SET_HIRQ2_SEL_OFF		TPDAS_A65M_set_hirq2_sel(TPDAS_A65M_HIRQ_SEL_NO)
#define TPDAS_A65M_SET_HIRQ2_EN_OFF		TPDAS_A65M_set_hirq2_en(TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ2_EN_ON		TPDAS_A65M_set_hirq2_en(TPDAS_A65M_HIRQ_EN_ON)


	//-- HIRQ-3 Interrupt control Register
#define TPDAS_A65M_hirq3_sel_rg_data()		((*(unsigned *)TPDAS_A65M_HIRQ3_SEL_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_hirq3_sel_rg(d)		(*(unsigned *)TPDAS_A65M_HIRQ3_SEL_RG) = d

#define TPDAS_A65M_hirq3_sel_data()		(TPDAS_A65M_hirq3_sel_rg_data() & TPDAS_A65M_HIRQ_SEL_BITMASK)
#define TPDAS_A65M_set_hirq3_sel(d)		TPDAS_A65M_set_hirq3_sel_rg((TPDAS_A65M_hirq3_sel_rg_data() & ~TPDAS_A65M_HIRQ_SEL_BITMASK) |  (d & TPDAS_A65M_HIRQ_SEL_BITMASK))
#define TPDAS_A65M_hirq3_en_data()		(TPDAS_A65M_hirq3_sel_rg_data() & TPDAS_A65M_HIRQ_EN_BITMASK)
#define TPDAS_A65M_set_hirq3_en(d)		TPDAS_A65M_set_hirq3_sel_rg((TPDAS_A65M_hirq3_sel_rg_data() & ~TPDAS_A65M_HIRQ_EN_BITMASK) |  (d & TPDAS_A65M_HIRQ_EN_BITMASK))

#define TPDAS_A65M_HIRQ3_SEL_RG_DATA		TPDAS_A65M_hirq3_sel_rg_data()
#define TPDAS_A65M_SET_HIRQ3_SEL_RG(d)		TPDAS_A65M_set_hirq3_sel_rg(d)
#define TPDAS_A65M_HIRQ3_SEL_DATA		TPDAS_A65M_hirq3_sel_data()
#define TPDAS_A65M_SET_HIRQ3_SEL(d)		TPDAS_A65M_set_hirq3_sel(d)
#define TPDAS_A65M_HIRQ3_EN_DATA		TPDAS_A65M_hirq3_en_data()
#define TPDAS_A65M_SET_HIRQ3_EN(d)		TPDAS_A65M_set_hirq3_en(d)

		//~~~ setting macros
#define TPDAS_A65M_SET_HIRQ3_OFF		TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_NO | TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ3_ADC_DAQ_END	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_DAC_DAQ_END	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_END  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_ADC_DAQ_SYNC	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_DAC_DAQ_SYNC	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_DAQ_SYNC | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_ADC_IRT_POS	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_DAC_IRT_POS	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_POS  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_ADC_IRT_NEG	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_ADC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_DAC_IRT_NEG	TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_DAC_IRT_NEG  | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_GPIO0		TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO0        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_GPIO1		TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_GPIO1        | TPDAS_A65M_HIRQ_EN_ON)
#define TPDAS_A65M_SET_HIRQ3_ERR		TPDAS_A65M_set_hirq3_sel_rg(TPDAS_A65M_HIRQ_SEL_ERR          | TPDAS_A65M_HIRQ_EN_ON)

#define TPDAS_A65M_SET_HIRQ3_SEL_OFF		TPDAS_A65M_set_hirq3_sel(TPDAS_A65M_HIRQ_SEL_NO)
#define TPDAS_A65M_SET_HIRQ3_EN_OFF		TPDAS_A65M_set_hirq3_en(TPDAS_A65M_HIRQ_EN_OFF)
#define TPDAS_A65M_SET_HIRQ3_EN_ON		TPDAS_A65M_set_hirq3_en(TPDAS_A65M_HIRQ_EN_ON)


	//-- HIRQ ERR Aux Interrupt Mask (XIM) Register
#define TPDAS_A65M_xim_err_rg_data()		((*(unsigned *)TPDAS_A65M_XIM_ERR_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_xim_err_rg(d)		(*(unsigned *)TPDAS_A65M_XIM_ERR_RG) = d

#define TPDAS_A65M_xim_adc_fifo_ovf_data()	(TPDAS_A65M_xim_err_rg_data() & TPDAS_A65M_XIM_ADC_FIFO_OVF_BITMASK)
#define TPDAS_A65M_set_xim_adc_fifo_ovf(d)	TPDAS_A65M_set_xim_err_rg((TPDAS_A65M_xim_err_rg_data() & ~TPDAS_A65M_XIM_ADC_FIFO_OVF_BITMASK) |  (d & TPDAS_A65M_XIM_ADC_FIFO_OVF_BITMASK))
#define TPDAS_A65M_xim_adc_fifo_unf_data()	(TPDAS_A65M_xim_err_rg_data() & TPDAS_A65M_XIM_ADC_FIFO_UNF_BITMASK)
#define TPDAS_A65M_set_xim_adc_fifo_unf(d)	TPDAS_A65M_set_xim_err_rg((TPDAS_A65M_xim_err_rg_data() & ~TPDAS_A65M_XIM_ADC_FIFO_UNF_BITMASK) |  (d & TPDAS_A65M_XIM_ADC_FIFO_UNF_BITMASK))
#define TPDAS_A65M_xim_dac_fifo_ovf_data()	(TPDAS_A65M_xim_err_rg_data() & TPDAS_A65M_XIM_DAC_FIFO_OVF_BITMASK)
#define TPDAS_A65M_set_xim_dac_fifo_ovf(d)	TPDAS_A65M_set_xim_err_rg((TPDAS_A65M_xim_err_rg_data() & ~TPDAS_A65M_XIM_DAC_FIFO_OVF_BITMASK) |  (d & TPDAS_A65M_XIM_DAC_FIFO_OVF_BITMASK))
#define TPDAS_A65M_xim_dac_fifo_unf_data()	(TPDAS_A65M_xim_err_rg_data() & TPDAS_A65M_XIM_DAC_FIFO_UNF_BITMASK)
#define TPDAS_A65M_set_xim_dac_fifo_unf(d)	TPDAS_A65M_set_xim_err_rg((TPDAS_A65M_xim_err_rg_data() & ~TPDAS_A65M_XIM_DAC_FIFO_UNF_BITMASK) |  (d & TPDAS_A65M_XIM_DAC_FIFO_UNF_BITMASK))
#define TPDAS_A65M_xim_adc_irt_err_data()	(TPDAS_A65M_xim_err_rg_data() & TPDAS_A65M_XIM_ADC_IRT_ERR_BITMASK)
#define TPDAS_A65M_set_xim_adc_irt_err(d)	TPDAS_A65M_set_xim_err_rg((TPDAS_A65M_xim_err_rg_data() & ~TPDAS_A65M_XIM_ADC_IRT_ERR_BITMASK) |  (d & TPDAS_A65M_XIM_ADC_IRT_ERR_BITMASK))
#define TPDAS_A65M_xim_dac_irt_err_data()	(TPDAS_A65M_xim_err_rg_data() & TPDAS_A65M_XIM_DAC_IRT_ERR_BITMASK)
#define TPDAS_A65M_set_xim_dac_irt_err(d)	TPDAS_A65M_set_xim_err_rg((TPDAS_A65M_xim_err_rg_data() & ~TPDAS_A65M_XIM_DAC_IRT_ERR_BITMASK) |  (d & TPDAS_A65M_XIM_DAC_IRT_ERR_BITMASK))

		//~~~ setting macros
#define TPDAS_A65M_SET_XIM_ADC_FIFO_OVF_OFF	TPDAS_A65M_set_xim_adc_fifo_ovf(TPDAS_A65M_XIM_ADC_FIFO_OVF_OFF)
#define TPDAS_A65M_SET_XIM_ADC_FIFO_OVF_ON	TPDAS_A65M_set_xim_adc_fifo_ovf(TPDAS_A65M_XIM_ADC_FIFO_OVF_ON)
#define TPDAS_A65M_SET_XIM_ADC_FIFO_UNF_OFF	TPDAS_A65M_set_xim_adc_fifo_unf(TPDAS_A65M_XIM_ADC_FIFO_UNF_OFF)
#define TPDAS_A65M_SET_XIM_ADC_FIFO_UNF_ON	TPDAS_A65M_set_xim_adc_fifo_unf(TPDAS_A65M_XIM_ADC_FIFO_UNF_ON)
#define TPDAS_A65M_SET_XIM_DAC_FIFO_OVF_OFF	TPDAS_A65M_set_xim_dac_fifo_ovf(TPDAS_A65M_XIM_DAC_FIFO_OVF_OFF)
#define TPDAS_A65M_SET_XIM_DAC_FIFO_OVF_ON	TPDAS_A65M_set_xim_dac_fifo_ovf(TPDAS_A65M_XIM_DAC_FIFO_OVF_ON)
#define TPDAS_A65M_SET_XIM_DAC_FIFO_UNF_OFF	TPDAS_A65M_set_xim_dac_fifo_unf(TPDAS_A65M_XIM_DAC_FIFO_UNF_OFF)
#define TPDAS_A65M_SET_XIM_DAC_FIFO_UNF_ON	TPDAS_A65M_set_xim_dac_fifo_unf(TPDAS_A65M_XIM_DAC_FIFO_UNF_ON)
#define TPDAS_A65M_SET_XIM_ADC_IRT_ERR_OFF	TPDAS_A65M_set_xim_adc_irt_err(TPDAS_A65M_XIM_ADC_IRT_ERR_OFF)
#define TPDAS_A65M_SET_XIM_ADC_IRT_ERR_ON	TPDAS_A65M_set_xim_adc_irt_err(TPDAS_A65M_XIM_ADC_IRT_ERR_ON)
#define TPDAS_A65M_SET_XIM_DAC_IRT_ERR_OFF	TPDAS_A65M_set_xim_adc_irt_err(TPDAS_A65M_XIM_DAC_IRT_ERR_OFF)
#define TPDAS_A65M_SET_XIM_DAC_IRT_ERR_ON	TPDAS_A65M_set_xim_adc_irt_err(TPDAS_A65M_XIM_DAC_IRT_ERR_ON)

		//~~~ macros for data aligned at bit #0
		//    input params: 
		//    -------------	
		//	TPDAS_A65M_XIM_OFF, TPDAS_A65M_XIM_ON
#define TPDAS_A65M_XIM_ADC_FIFO_OVF_DATA	TPDAS_A65M_xim_adc_fifo_ovf_data()
#define TPDAS_A65M_SET_XIM_ADC_FIFO_OVF(d)	TPDAS_A65M_set_xim_adc_fifo_ovf(d)
#define TPDAS_A65M_XIM_ADC_FIFO_UNF_DATA	(TPDAS_A65M_xim_adc_fifo_unf_data() >> TPDAS_A65M_XIM_ADC_FIFO_UNF_SHIFT)
#define TPDAS_A65M_SET_XIM_ADC_FIFO_UNF(d)	TPDAS_A65M_set_xim_adc_fifo_unf(d<<TPDAS_A65M_XIM_ADC_FIFO_UNF_SHIFT)
#define TPDAS_A65M_XIM_DAC_FIFO_OVF_DATA	(TPDAS_A65M_xim_dac_fifo_ovf_data() >> TPDAS_A65M_XIM_DAC_FIFO_OVF_SHIFT)
#define TPDAS_A65M_SET_XIM_DAC_FIFO_OVF(d)	TPDAS_A65M_set_xim_dac_fifo_ovf(d<<TPDAS_A65M_XIM_DAC_FIFO_OVF_SHIFT)
#define TPDAS_A65M_XIM_DAC_FIFO_UNF_DATA	(TPDAS_A65M_xim_dac_fifo_unf_data() >> TPDAS_A65M_XIM_DAC_FIFO_UNF_SHIFT)
#define TPDAS_A65M_SET_XIM_DAC_FIFO_UNF(d)	TPDAS_A65M_set_xim_dac_fifo_unf(d<<TPDAS_A65M_XIM_DAC_FIFO_UNF_SHIFT)
#define TPDAS_A65M_XIM_ADC_IRT_ERR_DATA		(TPDAS_A65M_xim_adc_irt_err_data() >> TPDAS_A65M_XIM_ADC_IRT_ERR_SHIFT)
#define TPDAS_A65M_SET_XIM_ADC_IRT_ERR(d)	TPDAS_A65M_set_xim_adc_irt_err(d<<TPDAS_A65M_XIM_ADC_IRT_ERR_SHIFT)
#define TPDAS_A65M_XIM_DAC_IRT_ERR_DATA		(TPDAS_A65M_xim_dac_irt_err_data() >> TPDAS_A65M_XIM_DAC_IRT_ERR_SHIFT)
#define TPDAS_A65M_SET_XIM_DAC_IRT_ERR(d)	TPDAS_A65M_set_xim_dac_irt_err(d<<TPDAS_A65M_XIM_DAC_IRT_ERR_SHIFT)


	//--- GPIO control registers
#define TPDAS_A65M_gpio_dir_rg_data()		((*(unsigned *)TPDAS_A65M_GPIO_DIR_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_gpio_dir_rg(d)		(*(unsigned *)TPDAS_A65M_GPIO_DIR_RG) = d

#define TPDAS_A65M_gpio0_dir_data()		(TPDAS_A65M_gpio_dir_rg_data() & TPDAS_A65M_GPIO0_DIR_BITMASK)
#define TPDAS_A65M_set_gpio0_dir(d)		TPDAS_A65M_set_gpio_dir_rg((TPDAS_A65M_gpio_dir_rg_data() & ~TPDAS_A65M_GPIO0_DIR_BITMASK) |  (d & TPDAS_A65M_GPIO0_DIR_BITMASK))
#define TPDAS_A65M_gpio1_dir_data()		(TPDAS_A65M_gpio_dir_rg_data() & TPDAS_A65M_GPIO1_DIR_BITMASK)
#define TPDAS_A65M_set_gpio1_dir(d)		TPDAS_A65M_set_gpio_dir_rg((TPDAS_A65M_gpio_dir_rg_data() & ~TPDAS_A65M_GPIO1_DIR_BITMASK) |  (d & TPDAS_A65M_GPIO1_DIR_BITMASK))
#define TPDAS_A65M_gpio2_dir_data()		(TPDAS_A65M_gpio_dir_rg_data() & TPDAS_A65M_GPIO2_DIR_BITMASK)
#define TPDAS_A65M_set_gpio2_dir(d)		TPDAS_A65M_set_gpio_dir_rg((TPDAS_A65M_gpio_dir_rg_data() & ~TPDAS_A65M_GPIO2_DIR_BITMASK) |  (d & TPDAS_A65M_GPIO2_DIR_BITMASK))
#define TPDAS_A65M_gpio3_dir_data()		(TPDAS_A65M_gpio_dir_rg_data() & TPDAS_A65M_GPIO3_DIR_BITMASK)
#define TPDAS_A65M_set_gpio3_dir(d)		TPDAS_A65M_set_gpio_dir_rg((TPDAS_A65M_gpio_dir_rg_data() & ~TPDAS_A65M_GPIO3_DIR_BITMASK) |  (d & TPDAS_A65M_GPIO3_DIR_BITMASK))

#define TPDAS_A65M_gpio_data_rg_data()		((*(unsigned *)TPDAS_A65M_GPIO_DATA_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_gpio_data_rg(d)		(*(unsigned *)TPDAS_A65M_GPIO_DATA_RG) = d

#define TPDAS_A65M_gpio0_data_data()		(TPDAS_A65M_gpio_data_rg_data() & TPDAS_A65M_GPIO0_DATA_BITMASK)
#define TPDAS_A65M_set_gpio0_data(d)		TPDAS_A65M_set_gpio_data_rg((TPDAS_A65M_gpio_data_rg_data() & ~TPDAS_A65M_GPIO0_DATA_BITMASK) |  (d & TPDAS_A65M_GPIO0_DATA_BITMASK))
#define TPDAS_A65M_gpio1_data_data()		(TPDAS_A65M_gpio_data_rg_data() & TPDAS_A65M_GPIO1_DATA_BITMASK)
#define TPDAS_A65M_set_gpio1_data(d)		TPDAS_A65M_set_gpio_data_rg((TPDAS_A65M_gpio_data_rg_data() & ~TPDAS_A65M_GPIO1_DATA_BITMASK) |  (d & TPDAS_A65M_GPIO1_DATA_BITMASK))
#define TPDAS_A65M_gpio2_data_data()		(TPDAS_A65M_gpio_data_rg_data() & TPDAS_A65M_GPIO2_DATA_BITMASK)
#define TPDAS_A65M_set_gpio2_data(d)		TPDAS_A65M_set_gpio_data_rg((TPDAS_A65M_gpio_data_rg_data() & ~TPDAS_A65M_GPIO2_DATA_BITMASK) |  (d & TPDAS_A65M_GPIO2_DATA_BITMASK))
#define TPDAS_A65M_gpio3_data_data()		(TPDAS_A65M_gpio_data_rg_data() & TPDAS_A65M_GPIO3_DATA_BITMASK)
#define TPDAS_A65M_set_gpio3_data(d)		TPDAS_A65M_set_gpio_data_rg((TPDAS_A65M_gpio_data_rg_data() & ~TPDAS_A65M_GPIO3_DATA_BITMASK) |  (d & TPDAS_A65M_GPIO3_DATA_BITMASK))

		//~~~ setting macros
#define TPDAS_A65M_SET_GPIO0_DIR_INPUT		TPDAS_A65M_set_gpio0_dir(TPDAS_A65M_GPIO0_DIR_INPUT)
#define TPDAS_A65M_SET_GPIO0_DIR_OUTPUT		TPDAS_A65M_set_gpio0_dir(TPDAS_A65M_GPIO0_DIR_OUTPUT)
#define TPDAS_A65M_SET_GPIO1_DIR_INPUT		TPDAS_A65M_set_gpio1_dir(TPDAS_A65M_GPIO1_DIR_INPUT)
#define TPDAS_A65M_SET_GPIO1_DIR_OUTPUT		TPDAS_A65M_set_gpio1_dir(TPDAS_A65M_GPIO1_DIR_OUTPUT)
#define TPDAS_A65M_SET_GPIO2_DIR_INPUT		TPDAS_A65M_set_gpio2_dir(TPDAS_A65M_GPIO2_DIR_INPUT)
#define TPDAS_A65M_SET_GPIO2_DIR_OUTPUT		TPDAS_A65M_set_gpio2_dir(TPDAS_A65M_GPIO2_DIR_OUTPUT)
#define TPDAS_A65M_SET_GPIO3_DIR_INPUT		TPDAS_A65M_set_gpio3_dir(TPDAS_A65M_GPIO3_DIR_INPUT)
#define TPDAS_A65M_SET_GPIO3_DIR_OUTPUT		TPDAS_A65M_set_gpio3_dir(TPDAS_A65M_GPIO3_DIR_OUTPUT)

#define TPDAS_A65M_SET_GPIO0_DATA_OFF		TPDAS_A65M_set_gpio0_data(TPDAS_A65M_GPIO0_DATA_OFF)
#define TPDAS_A65M_SET_GPIO0_DATA_ON		TPDAS_A65M_set_gpio0_data(TPDAS_A65M_GPIO0_DATA_ON)
#define TPDAS_A65M_SET_GPIO1_DATA_OFF		TPDAS_A65M_set_gpio1_data(TPDAS_A65M_GPIO1_DATA_OFF)
#define TPDAS_A65M_SET_GPIO1_DATA_ON		TPDAS_A65M_set_gpio1_data(TPDAS_A65M_GPIO1_DATA_ON)
#define TPDAS_A65M_SET_GPIO2_DATA_OFF		TPDAS_A65M_set_gpio2_data(TPDAS_A65M_GPIO2_DATA_OFF)
#define TPDAS_A65M_SET_GPIO2_DATA_ON		TPDAS_A65M_set_gpio2_data(TPDAS_A65M_GPIO2_DATA_ON)
#define TPDAS_A65M_SET_GPIO3_DATA_OFF		TPDAS_A65M_set_gpio3_data(TPDAS_A65M_GPIO3_DATA_OFF)
#define TPDAS_A65M_SET_GPIO3_DATA_ON		TPDAS_A65M_set_gpio3_data(TPDAS_A65M_GPIO3_DATA_ON)

		//~~~ macros for data aligned at bit #0
		//    input params: 
		//    -------------	
		//	DIR: 	TPDAS_A65M_GPIO_DIR_INPUT, TPDAS_A65M_GPIO_DIR_OUTPUT
		//	DATA:	TPDAS_A65M_GPIO_DATA_OFF, TPDAS_A65M_GPIO_DATA_ON
#define TPDAS_A65M_GPIO0_DIR_DATA		TPDAS_A65M_gpio0_dir_data()
#define TPDAS_A65M_SET_GPIO0_DIR(d)		TPDAS_A65M_set_gpio0_dir(d)
#define TPDAS_A65M_GPIO1_DIR_DATA	L	(TPDAS_A65M_gpio1_dir_data() >> TPDAS_A65M_GPIO1_DIR_SHIFT)
#define TPDAS_A65M_SET_GPIO1_DIR(d)		TPDAS_A65M_set_gpio1_dir(d<<TPDAS_A65M_GPIO1_DIR_SHIFT)
#define TPDAS_A65M_GPIO2_DIR_DATA		(TPDAS_A65M_gpio2_dir_data() >>TPDAS_A65M_GPIO2_DIR_SHIFT)
#define TPDAS_A65M_SET_GPIO2_DIR(d)		TPDAS_A65M_set_gpio2_dir(d<<TPDAS_A65M_GPIO2_DIR_SHIFT)
#define TPDAS_A65M_GPIO3_DIR_DATA		(TPDAS_A65M_gpio3_dir_data() >>TPDAS_A65M_GPIO3_DIR_SHIFT)
#define TPDAS_A65M_SET_GPIO3_DIR(d)		TPDAS_A65M_set_gpio3_dir(d<<TPDAS_A65M_GPIO3_DIR_SHIFT)

#define TPDAS_A65M_GPIO0_DATA_DATA		TPDAS_A65M_gpio0_data_data()
#define TPDAS_A65M_SET_GPIO0_DATA(d)		TPDAS_A65M_set_gpio0_data(d)
#define TPDAS_A65M_GPIO1_DATA_DATA		(TPDAS_A65M_gpio1_data_data() >> TPDAS_A65M_GPIO1_DATA_SHIFT)
#define TPDAS_A65M_SET_GPIO1_DATA(d)		TPDAS_A65M_set_gpio1_data(d << TPDAS_A65M_GPIO1_DATA_SHIFT)
#define TPDAS_A65M_GPIO2_DATA_DATA		(TPDAS_A65M_gpio2_data_data() >> TPDAS_A65M_GPIO2_DATA_SHIFT)
#define TPDAS_A65M_SET_GPIO2_DATA(d)		TPDAS_A65M_set_gpio2_data(d << TPDAS_A65M_GPIO2_DATA_SHIFT)
#define TPDAS_A65M_GPIO3_DATA_DATA		(TPDAS_A65M_gpio3_data_data() >> TPDAS_A65M_GPIO3_DATA_SHIFT)
#define TPDAS_A65M_SET_GPIO3_DATA(d)		TPDAS_A65M_set_gpio3_data(d << TPDAS_A65M_GPIO3_DATA_SHIFT)


	//--- PFG CNTR1..3 registers
#define TPDAS_A65M_pfg_cntr1_rg_data()		((*(unsigned *)TPDAS_A65M_PFG_CNTR1_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_pfg_cntr1_rg(d)		(*(unsigned *)TPDAS_A65M_PFG_CNTR1_RG) = d
#define TPDAS_A65M_pfg_cntr2_rg_data()		((*(unsigned *)TPDAS_A65M_PFG_CNTR2_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_pfg_cntr2_rg(d)		(*(unsigned *)TPDAS_A65M_PFG_CNTR2_RG) = d
#define TPDAS_A65M_pfg_cntr3_rg_data()		((*(unsigned *)TPDAS_A65M_PFG_CNTR3_RG) & TPDAS_A65M_CNTR_RG_DATA_BITMASK)
#define TPDAS_A65M_set_pfg_cntr3_rg(d)		(*(unsigned *)TPDAS_A65M_PFG_CNTR3_RG) = d

#define TPDAS_A65M_pfg_v_data()			((TPDAS_A65M_pfg_cntr1_rg_data() & TPDAS_A65M_PFG_V07_BITMASK) | ((TPDAS_A65M_pfg_cntr2_rg_data() & TPDAS_A65M_PFG_V8_BITMASK)<<8))
#define TPDAS_A65M_set_pfg_v(d)			{TPDAS_A65M_set_pfg_cntr1_rg(d & 0xff); TPDAS_A65M_set_pfg_cntr2_rg((TPDAS_A65M_pfg_cntr2_rg_data() & ~TPDAS_A65M_PFG_V8_BITMASK) |  ((d>>8) & TPDAS_A65M_PFG_V8_BITMASK));}
#define TPDAS_A65M_pfg_r_data()			(TPDAS_A65M_pfg_cntr2_rg_data() & TPDAS_A65M_PFG_R_BITMASK)
#define TPDAS_A65M_set_pfg_r(d)			TPDAS_A65M_set_pfg_cntr2_rg((TPDAS_A65M_pfg_cntr2_rg_data() & ~TPDAS_A65M_PFG_R_BITMASK) |  (d & TPDAS_A65M_PFG_R_BITMASK))
#define TPDAS_A65M_pfg_s_data()			(TPDAS_A65M_pfg_cntr3_rg_data() & TPDAS_A65M_PFG_S_BITMASK)
#define TPDAS_A65M_set_pfg_s(d)			TPDAS_A65M_set_pfg_cntr3_rg((TPDAS_A65M_pfg_cntr3_rg_data() & ~TPDAS_A65M_PFG_S_BITMASK) |  (d & TPDAS_A65M_PFG_S_BITMASK))
#define TPDAS_A65M_pfg_x_data()			(TPDAS_A65M_pfg_cntr3_rg_data() & TPDAS_A65M_PFG_X_BITMASK)
#define TPDAS_A65M_set_pfg_x(d)			TPDAS_A65M_set_pfg_cntr3_rg((TPDAS_A65M_pfg_cntr3_rg_data() & ~TPDAS_A65M_PFG_X_BITMASK) |  (d & TPDAS_A65M_PFG_X_BITMASK))

		//~~~ setting macros
#define TPDAS_A65M_SET_PFG_X_DIV1		TPDAS_A65M_set_pfg_x(TPDAS_A65M_X_DIV1)
#define TPDAS_A65M_SET_PFG_X_DIV16		TPDAS_A65M_set_pfg_x(TPDAS_A65M_X_DIV16)

#define TPDAS_A65M_SET_PFG_S_DIV2		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV2)
#define TPDAS_A65M_SET_PFG_S_DIV4		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV4)
#define TPDAS_A65M_SET_PFG_S_DIV5		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV5)
#define TPDAS_A65M_SET_PFG_S_DIV6		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV6)
#define TPDAS_A65M_SET_PFG_S_DIV7		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV7)
#define TPDAS_A65M_SET_PFG_S_DIV8		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV8)
#define TPDAS_A65M_SET_PFG_S_DIV9		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV9)
#define TPDAS_A65M_SET_PFG_S_DIV10		TPDAS_A65M_set_pfg_s(TPDAS_A65M_S_DIV10)

		//~~~ macros for data aligned at bit #0
		//    V/R/S field data are aligned at bit #0 in order to simplify passing of numeric values.
#define TPDAS_A65M_PFG_V_DATA			TPDAS_A65M_pfg_v_data()
#define	TPDAS_A65M_SET_PFG_V(d)			TPDAS_A65M_set_pfg_v(d)
#define	TPDAS_A65M_PFG_R_DATA			(TPDAS_A65M_pfg_r_data() >>TPDAS_A65M_PFG_R_SHIFT)
#define	TPDAS_A65M_SET_PFG_R(d)			TPDAS_A65M_set_pfg_r(d<<TPDAS_A65M_PFG_R_SHIFT)
#define TPDAS_A65M_PFG_S_DATA			TPDAS_A65M_pfg_s_data()
#define TPDAS_A65M_SET_PFG_S(d)			TPDAS_A65M_set_pfg_s(d)
#define TPDAS_A65M_PFG_X_DATA			TPDAS_A65M_pfg_x_data()
#define TPDAS_A65M_SET_PFG_X(d)			TPDAS_A65M_set_pfg_x(d)

#define TPDAS_A65M_SET_PFG(v,r,s,x)		{TPDAS_A65M_SET_PFG_V(v);TPDAS_A65M_SET_PFG_R(r);TPDAS_A65M_SET_PFG_S(s);TPDAS_A65M_SET_PFG_X(x);}

		//~~~ macros to set PFG V/R/S/X fields, which are updated by PFG buld in +8 and +2 shifters for V and R fields
#define TPDAS_A65M_SET_PFG_EXACTLY(v,r,s,x)	{TPDAS_A65M_SET_PFG_V(v-8);TPDAS_A65M_SET_PFG_R(r-2);TPDAS_A65M_SET_PFG_S(s);TPDAS_A65M_SET_PFG_X(x);}


//=====================================================================


//========== macros for accessing ADC/DAC FIFO data  ==================

	//~~~ programming ADC/DAC FIFO PAF/PAE flags in CNF mode
#define TPDAS_A65M_set_adc_fifo_sin(d)	(*(unsigned *)TPDAS_A65M_ADC_FIFO_SIN_RG) = d
#define TPDAS_A65M_set_dac_fifo_sin(d)	(*(unsigned *)TPDAS_A65M_DAC_FIFO_SIN_RG) = d


	//~~~ ADC/DAC FIFO data access macros
#define TPDAS_A65M_adc1_fifo_rg_data()	((*(unsigned *)TPDAS_A65M_ADC1_FIFO_DATA_RG) & TPDAS_A65M_XDS_DATA_BITMASK)
#define TPDAS_A65M_adc2_fifo_rg_data()	((*(unsigned *)TPDAS_A65M_ADC2_FIFO_DATA_RG) & TPDAS_A65M_XDS_DATA_BITMASK)

#define TPDAS_A65M_adc1_data()		(TPDAS_A65M_adc1_fifo_rg_data() & TPDAS_A65M_ADDA_DATA_BITMASK)
#define TPDAS_A65M_adc2_data()		(TPDAS_A65M_adc2_fifo_rg_data() & TPDAS_A65M_ADDA_DATA_BITMASK)
#define TPDAS_A65M_adc_rimux_data()	(TPDAS_A65M_adc1_fifo_rg_data() & TPDAS_A65M_ADC_RIMUX_BITMASK)
#define TPDAS_A65M_xdsin1_data()	(TPDAS_A65M_adc1_fifo_rg_data() & TPDAS_A65M_XDS_DATA_BITMASK)
#define TPDAS_A65M_xdsin2_data()	(TPDAS_A65M_adc2_fifo_rg_data() & TPDAS_A65M_XDS_DATA_BITMASK)

#define TPDAS_A65M_set_dac1_fifo_rg(d)	(*(unsigned *)TPDAS_A65M_DAC1_FIFO_DATA_RG)=d
#define TPDAS_A65M_set_dac2_fifo_rg(d)	(*(unsigned *)TPDAS_A65M_DAC2_FIFO_DATA_RG)=d

#define TPDAS_A65M_set_dac1(d)		TPDAS_A65M_set_dac1_fifo_rg(d)
#define TPDAS_A65M_set_dac2(d)		TPDAS_A65M_set_dac2_fifo_rg(d)
#define TPDAS_A65M_set_xdsout1(d)	TPDAS_A65M_set_dac1_fifo_rg(d)
#define TPDAS_A65M_set_xdsout2(d)	TPDAS_A65M_set_dac2_fifo_rg(d)

	//~~~ macros for data aligned at bit#0
	//    The parameter passing is used in order to excluded repeative reads of FIFO, which 
	//    effects FIFO read pointer. The parameter passed can be pre-readed FIFO data.
	//	returned data:
	//	-------------
	// 	  ADC RIMUX:	TPDAS_A65M_ADC_IMUX_CH0/1/2/3
	// 	  ADC DATA:	sign extended to 32-bits and right aligned
#define TPDAS_A65M_ADC_RIMUX_DATA(r)	(((r) & TPDAS_A65M_ADC_RIMUX_BITMASK) >> TPDAS_A65M_ADC_RIMUX_SHIFT)
#define TPDAS_A65M_ADC_DATA(r)		(((r) << 20) >>20)

//========================================================================



//******** optional functions declarations for PA65X_HD.H file ***********

int TPDAS_A65M_set_adc_fifo_offset(unsigned pae, unsigned paf);
int TPDAS_A65M_set_dac_fifo_offset(unsigned pae, unsigned paf);

//************************************************************************

